#!/usr/bin/env python3
# 
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#

import os, re

from qiling.const import *
from qiling.exception import *


from unicorn import (
    UC_PROT_ALL,
    UC_PROT_EXEC,
    UC_PROT_NONE,
    UC_PROT_READ,
    UC_PROT_WRITE,
)

class QlMemoryManager:
    """
    some ideas and code from:
    https://github.com/zeropointdynamics/zelos/blob/master/src/zelos/memory.py
    """

    def __init__(self, ql):
        self.ql = ql
        self.map_info = []

        bit_stuff = {
            64 : (0xFFFFFFFFFFFFFFFF,),
            32 : (0xFFFFFFFF,),
            16 : (0xFFFFF,)             # 20bit address line
        }

        if ql.archbit not in bit_stuff:
            raise QlErrorStructConversion("Unsupported Qiling archtecture for memory manager")

        max_addr, = bit_stuff[ql.archbit]

        #self.read_ptr = read_ptr
        self.max_addr = max_addr
        self.max_mem_addr = max_addr


    def string(self, addr, value=None ,encoding='utf-8'): 
        if value == None:
            ret = ""
            c = self.read(addr, 1)[0]
            read_bytes = 1

            while c != 0x0:
                ret += chr(c)
                c = self.read(addr + read_bytes, 1)[0]
                read_bytes += 1
            return ret
        else:
            string_bytes = bytes(value, encoding) + b'\x00'
            self.write(addr, string_bytes)
            return None

    def add_mapinfo(self, mem_s, mem_e, mem_p, mem_info):
        tmp_map_info = []
        insert_flag = 0
        map_info = self.map_info
        if len(map_info) == 0:
            tmp_map_info.append([mem_s, mem_e, mem_p, mem_info])
        else:
            for s, e, p, info in map_info:
                if e <= mem_s:
                    tmp_map_info.append([s, e, p, info])
                    continue
                if s >= mem_e:
                    if insert_flag == 0:
                        insert_flag = 1
                        tmp_map_info.append([mem_s, mem_e, mem_p, mem_info])
                    tmp_map_info.append([s, e, p, info])
                    continue
                if s < mem_s:
                    tmp_map_info.append([s, mem_s, p, info])

                if s == mem_s:
                    pass

                if insert_flag == 0:
                    insert_flag = 1
                    tmp_map_info.append([mem_s, mem_e, mem_p, mem_info])

                if e > mem_e:
                    tmp_map_info.append([mem_e, e, p, info])

                if e == mem_e:
                    pass
            if insert_flag == 0:
                tmp_map_info.append([mem_s, mem_e, mem_p, mem_info])

        self.map_info = tmp_map_info


    def del_mapinfo(self, mem_s, mem_e):
        tmp_map_info = []

        for s, e, p, info in self.map_info:
            if e <= mem_s:
                tmp_map_info.append([s, e, p, info])
                continue

            if s >= mem_e:
                tmp_map_info.append([s, e, p, info])
                continue

            if s < mem_s:
                tmp_map_info.append([s, mem_s, p, info])

            if s == mem_s:
                pass

            if e > mem_e:
                tmp_map_info.append([mem_e, e, p, info])

            if e == mem_e:
                pass

        self.map_info = tmp_map_info


    def show_mapinfo(self):
        def _perms_mapping(ps):
            perms_d = {1: "r", 2: "w", 4: "x"}
            perms_sym = []
            for idx, val in perms_d.items():
                if idx & ps != 0:
                    perms_sym.append(val)
                else:
                    perms_sym.append("-")
            return "".join(perms_sym)

        self.ql.log.info("[+] Start      End        Perm.  Path")
        for  start, end, perm, info in self.map_info:
            _perm = _perms_mapping(perm)
            image = self.ql.os.find_containing_image(start)
            if image:
                info += f" ({image.path})"
            self.ql.log.info("[+] %08x - %08x - %s    %s" % (start, end, _perm, info))


    def get_lib_base(self, filename):
        for s, e, p, info in self.map_info:
            if os.path.split(info)[1] == filename:
                return s
        return -1


    def align(self, addr, alignment=0x1000):
        # rounds up to nearest alignment
        mask = ((1 << self.ql.archbit) - 1) & -alignment
        return (addr + (alignment - 1)) & mask

    # save all mapped mem
    def save(self):
        mem_dict = {}
        seq = 1
        for start, end, perm, info in self.map_info:
            mem_read = self.read(start, end-start)          
            mem_dict[seq] = start, end, perm, info, mem_read
            seq += 1
        return mem_dict

    # restore all dumped memory
    def restore(self, mem_dict):
        for key, value in mem_dict.items():
            start = value[0]
            end = value[1]
            perm = value[2]
            info = value[3]
            mem_read = bytes(value[4])

            self.ql.log.debug("restore key: %i 0x%x 0x%x %s" % (key, start, end, info))
            if self.is_mapped(start, end-start) == False:
                self.ql.log.debug("mapping 0x%x 0x%x mapsize 0x%x" % (start, end, end-start))
                self.map(start, end-start, perms=perm, info=info)

            self.ql.log.debug("writing 0x%x size 0x%x write_size 0x%x " % (start, end-start, len(mem_read)))
            self.write(start, mem_read)

    def read(self, addr: int, size: int) -> bytearray:
        return self.ql.uc.mem_read(addr, size)

    def read_ptr(self, addr: int, size: int=None):
        if not size:
            size = self.ql.archbit // 8

        if size == 2:
            return self.ql.unpack16(self.read(addr, 2))
        elif size == 4:
            return self.ql.unpack32(self.read(addr, 4))
        elif size == 8:
            return self.ql.unpack64(self.read(addr, 8))
        else:
            raise QlErrorStructConversion(f"Unsupported pointer size: {size}")

    def write(self, addr: int, data: bytes) -> None:
        try:
            self.ql.uc.mem_write(addr, data)
        except:
            self.show_mapinfo()
            self.ql.log.debug("addresss write length: " + str(len(data)))
            self.ql.log.error("addresss write error: " + hex(addr))
            raise

    def search(self, needle: bytes, begin= None, end= None):
        """
        Search for a sequence of bytes in memory. Returns all sequences
        that match
        """

        addrs = []
        if (begin and end) and end > begin:
            haystack = self.read(begin, end - begin)
            addrs = [x.start(0) + begin for x in re.finditer(needle, haystack)]

        if not begin:
            begin = self.map_info[0][0] # search from the first mapped region 
        if not end:
            end = self.map_info[-1][1] # search till the last mapped region

        mapped_range = [(_begin, _end) for _begin, _end, _ in self.ql.uc.mem_regions()
                if _begin in range(begin, end) or _end in range(begin, end)]
        
        for _begin, _end in mapped_range:
            haystack = self.read(_begin, _end - _begin)
            addrs += [x.start(0) + _begin for x in re.finditer(needle, haystack)]

        return addrs

    def unmap(self, addr, size) -> None:
        '''
        The main function of mem_unmap is to reclaim memory.
        This function will reclaim the memory starting with addr and length of size.
        Upon successful completion, munmap() shall return 0; 
        otherwise, it shall return -1 and set errno to indicate the error.
        '''
        self.del_mapinfo(addr, addr + size)
        self.ql.uc.mem_unmap(addr, size)


    def unmap_all(self):
        for region in list(self.ql.uc.mem_regions()):
            if region[0] and region[1]:
                return self.unmap(region[0], ((region[1] - region[0])+0x1))


    def is_available(self, addr, size):
        '''
        The main function of is_available is to determine 
        whether the memory starting with addr and having a size of length can be used for allocation.

        If it can be allocated, returns True.

        If it cannot be allocated, it returns False.
        '''
        try:
            self.map(addr, addr)
        except:
            return False    
        
        self.unmap(addr, addr)
        return True


    def is_mapped(self, address, size): 
        '''
        The main function of is_mmaped is to determine 
        whether the memory starting with addr and size has been mapped.
        Returns true if it has already been allocated.
        If unassigned, returns False.
        '''   

        for region in list(self.ql.uc.mem_regions()):
            if address >= region[0] and (address + size -1) <= region[1]:
                return True

        return False
        
    
    def is_free(self, address, size):
        '''
        The main function of is_free first must fufull is_mapped condition.
        then, check for is the mapped range empty, either fill with 0xFF or 0x00
        Returns true if mapped range is empty else return Flase
        If not not mapped, map it and return true
        '''
        if self.is_mapped(address, size) == True:
            address_end = (address + size)
            while address < address_end:
                mem_read = self.ql.mem.read(address, 0x1)
                if (mem_read[0] != 0x00) and (mem_read[0] != 0xFF):
                    return False
                address += 1
            return True
        else:
            return True


    def find_free_space(
        self, size, min_addr=0, max_addr = 0, alignment=0x10000
    ):
        """
        Finds a region of memory that is free, larger than 'size' arg,
        and aligned.
        """
        mapped = []
        
        for address_start, address_end, perm, info in self.ql.mem.map_info:
            mapped += [[address_start, (address_end - address_start)]]
        
        for address_start, address_end, perms in self.ql.uc.mem_regions():
            mapped += [[address_start, (address_end - address_start)]]
        
        for i in range(0, len(mapped)):
            addr = self.align(
                mapped[i][0] + mapped[i][1], alignment=alignment
            )
            # Enable allocating memory in the middle of a gap when the
            # min requested address falls in the middle of a gap
            if addr < min_addr:
                addr = min_addr
            # Cap the gap's max address by accounting for the next
            # section's start address, requested max address, and the
            # max possible address

            max_gap_addr = (
                self.max_addr
                if i == len(mapped) - 1
                else mapped[i + 1][1]
            )

            max_gap_addr = min(max_gap_addr, self.max_mem_addr)
            # Ensure the end address is less than the max and the start
            # address is free
            if addr + size < max_gap_addr and self.is_mapped(addr, size) == False:
                return addr
        raise QlOutOfMemory("Out Of Memory")


    def map_anywhere(
        self,
        size,
        #name = "",
        #kind = "",
        min_addr = 0,
        alignment = 0x1000,
        #prot: int = ProtType.RWX,
    ) -> int:
        """
        Maps a region of memory with requested size, within the
        addresses specified. The size and start address will respect the
        alignment.

        Args:
            size: # of bytes to map. This will be rounded up to match
                the alignment.
            name: String used to identify mapped region. Used for
                debugging.
            kind: String used to identify the purpose of the mapped
                region. Used for debugging.
            min_addr: The lowest address that could be mapped.
            max_addr: The highest address that could be mapped.
            alignment: Ensures the size and start address are multiples
                of this. Must be a multiple of 0x1000. Default 0x1000.
            prot: RWX permissions of the mapped region. Defaults to
                granting all permissions.
        Returns:
            Start address of mapped region.
        """
        max_mem_addr = self.max_mem_addr
        address = self.find_free_space(
            size, min_addr=min_addr, max_addr=max_mem_addr, alignment=alignment
        )
        """
        we need a better mem_map as defined in the issue
        """
        #self.map(address, util.align(size), name, kind)
        self.map(address, self.align(size))
        return address

    def protect(self, addr, size, perms):
        aligned_address = addr & 0xFFFFF000  # Address needs to align with
        aligned_size = self.align((addr & 0xFFF) + size)
        self.ql.uc.mem_protect(aligned_address, aligned_size, perms)


    def map(self, addr, size, perms=UC_PROT_ALL, info=None, ptr=None):
        '''
	    The main function of mem_mmap is to implement memory allocation in unicorn, 
	    which is slightly similar to the function of syscall_mmap. 

	    When the memory can satisfy the given addr and size, 
	    it needs to be allocated to the corresponding address space.
	    
	    Upon successful completion, mem_map() shall return 0; 

    	otherwise, it shall return -1 and set errno to indicate the error.
         
        is should call other API to get_available mainly gives a length, 
        and then the memory manager returns  an address that can apply for that length.

        '''
        if ptr == None:
            if self.is_mapped(addr, size) == False:
                self.ql.uc.mem_map(addr, size, perms)
                self.add_mapinfo(addr, addr + size, perms, info if info else "[mapped]")
            else:
                raise QlMemoryMappedError("Memory Mapped")    
        else:
            self.ql.uc.mem_map_ptr(addr, size, perms, ptr)

    def get_mapped(self):
        for idx, val in enumerate(self.ql.uc.mem_regions()):
            self.ql.log.info(idx, list(map(hex, val)))

# A Simple Heap Implementation
class Chunk():
    def __init__(self, address, size):
        self.inuse = True
        self.address = address
        self.size = size

    @staticmethod
    def compare(chunk):
        return chunk.size

class QlMemoryHeap:
    def __init__(self, ql, start_address, end_address):
        self.ql = ql
        self.chunks = []
        self.start_address = start_address
        self.end_address = end_address
        # unicorn needs 0x1000
        self.page_size = 0x1000
        # current alloced memory size
        self.current_alloc = 0
        # curent use memory size
        self.current_use = 0
        # save all memory regions allocated
        self.mem_alloc = []
    
    def save(self):
        saved_state = {}
        saved_state['chunks'] = self.chunks
        saved_state['start_address'] = self.start_address
        saved_state['end_address'] = self.end_address
        saved_state['page_size'] = self.page_size
        saved_state['current_alloc'] = self.current_alloc
        saved_state['current_use'] = self.current_use
        saved_state['mem_alloc'] = self.mem_alloc
        return saved_state

    def restore(self, saved_state):
        self.chunks = saved_state['chunks']
        self.start_address = saved_state['start_address']
        self.end_address = saved_state['end_address']
        self.page_size = saved_state['page_size']
        self.current_alloc = saved_state['current_alloc']
        self.current_use = saved_state['current_use']
        self.mem_alloc = saved_state['mem_alloc']

    def alloc(self, size):
        # Find the heap chunks that best matches size 
        self.chunks.sort(key=Chunk.compare)
        for chunk in self.chunks:
            if chunk.inuse is False and chunk.size > size:
                chunk.inuse = True
                return chunk.address

        chunk = None
        # If we need mem_map new memory
        if self.current_use + size > self.current_alloc:
            real_size = self.ql.mem.align(size, self.page_size)
            # If the heap is not enough
            if self.start_address + self.current_use + real_size > self.end_address:
                return 0
            self.ql.mem.map(self.start_address + self.current_alloc, real_size, info="[heap]")
            chunk = Chunk(self.start_address + self.current_use, size)
            self.mem_alloc.append((self.start_address + self.current_alloc, real_size))
            self.current_alloc += real_size
            self.current_use += size
            self.chunks.append(chunk)
        else:
            chunk = Chunk(self.start_address + self.current_use, size)
            self.current_use += size
            self.chunks.append(chunk)

        chunk.inuse = True
        #ql.log.debug("heap.alloc addresss: " + hex(chunk.address))
        return chunk.address

    def size(self, addr):
        for chunk in self.chunks:
            if addr == chunk.address and chunk.inuse:
                return chunk.size
        return 0

    def free(self, addr):
        for chunk in self.chunks:
            if addr == chunk.address and chunk.inuse:
                chunk.inuse = False
                return True
        return False

    # clear all memory regions alloc
    def clear(self):
        for chunk in self.chunks:
            chunk.inuse = False

        for addr, size in self.mem_alloc:
            self.ql.mem.unmap(addr, size)

        self.mem_alloc.clear()

        self.current_alloc = 0
        self.current_use = 0

    def _find(self, addr):
        for chunk in self.chunks:
            if addr == chunk.address:
                return chunk
        return None



#!/usr/bin/env python3
#
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#


from qiling.const import *
from qiling.os.linux.thread import *
from qiling.const import *
from qiling.os.posix.filestruct import *
from qiling.os.filestruct import *
from qiling.os.posix.const_mapping import *
from qiling.exception import *

def ql_syscall_ptrace(ql, request, pid, addr, data, *args, **kw):
    regreturn = 0
    return regreturn



#!/usr/bin/env python3
#
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#

import struct
import time
from qiling.os.windows.const import *
from qiling.os.const import *
from qiling.os.windows.fncc import *
from qiling.os.windows.utils import *
from qiling.os.windows.thread import *
from qiling.os.windows.handle import *
from qiling.exception import *
from qiling.const import *


dllname = 'kernel32_dll'

# HANDLE CreateToolhelp32Snapshot(
#   DWORD dwFlags,
#   DWORD th32ProcessID
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_CreateToolhelp32Snapshot(ql, address, params):
    # TODO thinking about implementing an handler, gonna see if is really necessary
    flag = params["dwFlags"]
    if flag == TH32CS_SNAPPROCESS:
        ql.log.debug("The target is checking every process!")
    else:
        raise QlErrorNotImplemented("API not implemented")
    return 0xD10C


# BOOL Process32FirstW(
#   HANDLE            hSnapshot,
#   LPPROCESSENTRY32W lppe
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_Process32FirstW(ql, address, params):
    return 0x1


# BOOL Process32NextW(
#   HANDLE            hSnapshot,
#   LPPROCESSENTRY32W lppe
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_Process32NextW(ql, address, params):
    # Return True if more process, 0 else
    if ql.os.syscall_count["Process32NextW"] >= 3:  # I don' know how many process the sample want's to cycle
        return 0x0
    return 0x1



#!/usr/bin/env python3
#
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#

import gevent, os, time

from typing import Callable
from abc import ABC, abstractmethod
from pathlib import Path
from gevent import Greenlet
from unicorn.unicorn import UcError
from unicorn.mips_const import *
from unicorn.arm_const import *

from qiling.utils import FMT_STR
from qiling.os.thread import *
from qiling.arch.x86_const import *
from qiling.const import *
from qiling.os.const import *
from qiling.exception import *

LINUX_THREAD_ID = 2000


def new_thread_id():
    global LINUX_THREAD_ID
    old = LINUX_THREAD_ID
    LINUX_THREAD_ID += 1
    return old

class QlLinuxThread(QlThread):
    def __init__(self, ql, start_address, exit_point, context = None, set_child_tid_addr = None, thread_id = None):
        super(QlLinuxThread, self).__init__(ql)
        if not thread_id:
            self._thread_id = new_thread_id()
        else:
            self._thread_id = thread_id
        self._saved_context = context
        self._ql = ql
        self._exit_point = exit_point
        self._start_address = start_address
        self._status = THREAD_STATUS_RUNNING
        self._return_val = 0
        self._current_path = ql.os.current_path
        self._log_file_fd = None
        self._sched_cb = None

        # Compatibility
        self._log_file_fd = ql.log

        # For each thread, the kernel maintains two attributes (addresses)
        # called set_child_tid and clear_child_tid.  These two attributes
        # contain the value NULL by default.

        # set_child_tid
        #         If a thread is started using clone(2) with the
        #         CLONE_CHILD_SETTID flag, set_child_tid is set to the value
        #         passed in the ctid argument of that system call.

        #         When set_child_tid is set, the very first thing the new thread
        #         does is to write its thread ID at this address.

        # clear_child_tid
        #         If a thread is started using clone(2) with the
        #         CLONE_CHILD_CLEARTID flag, clear_child_tid is set to the value
        #         passed in the ctid argument of that system call.

        # The system call set_tid_address() sets the clear_child_tid value for
        # the calling thread to tidptr.

        # When a thread whose clear_child_tid is not NULL terminates, then, if
        # the thread is sharing memory with other threads, then 0 is written at
        # the address specified in clear_child_tid and the kernel performs the
        # following operation:

        #     futex(clear_child_tid, FUTEX_WAKE, 1, NULL, NULL, 0);

        # The effect of this operation is to wake a single thread that is
        # performing a futex wait on the memory location.  Errors from the
        # futex wake operation are ignored.

        # Source: Linux Man Page

        self._set_child_tid_address = set_child_tid_addr
        self._clear_child_tid_address = None

        self._robust_list_head_ptr = None
        self._robust_list_head_len = None

        if self._set_child_tid_address != None:
            self.ql.mem.write(self._set_child_tid_address, ql.pack32(self.id))

    @property
    def ql(self):
        return self._ql
    
    @ql.setter
    def ql(self, q):
        self._ql = q

    @property
    def saved_context(self):
        return self._saved_context

    @saved_context.setter
    def saved_context(self, ctx):
        self._saved_context = ctx

    @property
    def exit_point(self):
        return self._exit_point
    
    @exit_point.setter
    def exit_point(self, ep):
        self._exit_point = ep
    
    @property
    def start_address(self):
        return self._start_address
    
    @start_address.setter
    def start_address(self, sa):
        self._start_address = sa

    @property
    def status(self):
        return self._status
    
    @status.setter
    def status(self, s):
        self._status = s

    @property
    def return_val(self):
        return self._return_val

    @return_val.setter
    def return_val(self, rv):
        self._return_val = rv

    @property
    def current_path(self):
        return self._current_path

    @current_path.setter
    def current_path(self, cp):
        self._current_path = cp

    @property
    def log_file_fd(self):
        return self._log_file_fd

    @log_file_fd.setter
    def log_file_fd(self, lfd):
        self._log_file_fd = lfd

    @property
    def id(self):
        return self._thread_id

    def __hash__(self):
        return self.id
    
    def __str__(self):
        return f"[Thread {self.id}]"

    @property
    def set_child_tid_address(self):
        return self._set_child_tid_address
    
    @set_child_tid_address.setter
    def set_child_tid_address(self, addr):
        self._set_child_tid_address = addr
    
    @property
    def clear_child_tid_address(self):
        return self._clear_child_tid_address

    @clear_child_tid_address.setter
    def clear_child_tid_address(self, addr):
        self._clear_child_tid_address = addr
    
    @property
    def robust_list_head_ptr(self):
        return self._robust_list_head_ptr
    
    @robust_list_head_ptr.setter
    def robust_list_head_ptr(self, p):
        self._robust_list_head_ptr = p
    
    @property
    def robust_list_head_len(self):
        return self._robust_list_head_len
    
    @robust_list_head_len.setter
    def robust_list_head_len(self, l):
        self._robust_list_head_len = l

    @property
    def sched_cb(self) -> Callable:
        return self._sched_cb
    
    @sched_cb.setter
    def sched_cb(self, cb):
        self._sched_cb = cb

    def _default_sched_cb(self):
        # Give up control.
        gevent.sleep(0)

    def _run(self):
        # Some random notes for myself:
        # Implement details:
        #    The thread execution is divided in to two contexts:
        #        - Unicorn context.
        #        - Non-Unicorn context.
        #    Within both contexts, our program is single thread.
        #
        #    The only fail safe: **Never give up control in Unicorn context.**
        #    
        #    In Unicorn context, in other words, in Unicorn callbacks, we do:
        #        - Implement non-blocking syscalls directly.
        #        - Prepare sched_cb for non-unicorn context.
        #
        #    In Non-Unicorn context.
        #    In this context, we do:
        #        - Call gevent functions to switch threads.
        #        - Forward blocking syscalls to gevent.
        self.ql.reg.arch_pc = self.start_address
        if not self._saved_context:
            self.save()
        
        while self.status != THREAD_STATUS_TERMINATED:
            # Rewrite our status and the current thread.
            self.status = THREAD_STATUS_RUNNING
            self.ql.os.thread_management.cur_thread = self

            # Restore the context of the currently executing thread and set tls
            self.restore()

            # Sanity check
            if self.ql.reg.arch_pc == self.exit_point:
                self.ql.log.warning(f"Nothing to do but still get scheduled!")

            # Run and log the run event
            start_address = self.ql.arch.get_pc() # For arm thumb.
            self.sched_cb = QlLinuxThread._default_sched_cb
            
            self.ql.log.debug(f"Scheduled from {hex(start_address)}.")
            try:
                # Known issue for timeout: https://github.com/unicorn-engine/unicorn/issues/1355
                self.ql.emu_start(start_address, self.exit_point, count=30000)
            except UcError as e:
                self.ql.os.emu_error()
                self.ql.log.exception("")
                raise e
            self.ql.log.debug(f"Suspended at {hex(self.ql.reg.arch_pc)}")
            self.save()
            
            # Note that this callback may be set by UC callbacks.
            # Some thought on this design:
            #      1. Never give up control during a UC callback.
            #      2. emu_stop only sends a signal to unicorn which won't stop it immediately.
            #      3. According to 1, never call gevent functions in UC callbacks.
            self.ql.log.debug(f"Call sched_cb: {self.sched_cb}")
            self.sched_cb(self)

            if self.status == THREAD_STATUS_TERMINATED or self.ql.reg.arch_pc == self.exit_point:
                break

        self._on_stop()

    # Depreciated.
    def get_id(self):
        return self.id

    @abstractmethod
    def save(self):
        pass

    @abstractmethod
    def restore(self):
        pass

    @abstractmethod
    def set_thread_tls(self, tls_addr):
        pass
    
    @abstractmethod
    def clone(self):
        # This is a workaround to implement our thread based on gevent greenlet.
        # Core idea:
        #     A gevent greenlet can't re-run if it has finished _run method but our framework requires threads to be resumed anytime. Therefore, a workaround is to
        #     use multiple greenlets to represent a single qiling thread.
        #     
        #     Of course we can make the greenlet run forever and wait for notifications to resume but that would make the design much more complicated.
        #     
        # Caveat:
        #     Don't use thread id to identify the thread object.
        new_thread = self.ql.os.thread_class.spawn(self._ql, self._start_address, self._exit_point, self._saved_context, set_child_tid_addr = None, thread_id = self._thread_id)
        new_thread._current_path = self._current_path
        new_thread._return_val = self._return_val
        new_thread._robust_list_head_len = self._robust_list_head_len
        new_thread._robust_list_head_ptr = self._robust_list_head_ptr
        new_thread._set_child_tid_address = self._set_child_tid_address
        new_thread._clear_child_tid_address = self._clear_child_tid_address
        return new_thread

    def save_context(self):
        self.saved_context = self.ql.arch.context_save()

    def restore_context(self):
        self.ql.arch.context_restore(self.saved_context)

    def set_start_address(self, addr):
        # We can't modify UcContext directly.
        old_context = self.ql.arch.context_save()
        self.restore_context()
        self.ql.reg.arch_pc = addr
        self.save_context()
        self.ql.arch.context_restore(old_context)

    def set_clear_child_tid_addr(self, addr):
        self.clear_child_tid_address = addr

    def _on_stop(self):
        # CLONE_CHILD_CLEARTID (since Linux 2.5.49)
        #       Clear (zero) the child thread ID at the location pointed to by
        #       child_tid (clone()) or cl_args.child_tid (clone3()) in child
        #       memory when the child exits, and do a wakeup on the futex at
        #       that address.  The address involved may be changed by the
        #       set_tid_address(2) system call.  This is used by threading
        #       libraries.

        # Source: Linux Man Page

        if self.clear_child_tid_address is not None:
            self.ql.log.debug(f"Perform CLONE_CHILD_CLEARTID at {hex(self.clear_child_tid_address)}")
            self.ql.mem.write(self.clear_child_tid_address, self.ql.pack32(0))
            wakes = self.ql.os.futexm.get_futex_wake_list(self.ql, self.clear_child_tid_address, 1)
            self.clear_child_tid_address = None
            # When the thread is to stop, we don't have chance for next sched_cb, so
            # we notify the thread directly.
            for t, e in wakes:
                self.ql.log.debug(f"Notify {t}.")
                e.set()

    # This function should called outside unicorn callback.
    def stop(self):
        self.status = THREAD_STATUS_TERMINATED

    def is_stop(self):
        #return self.status == THREAD_STATUS_TERMINATED
        return self.dead

    def is_running(self):
        return not self.dead

    def is_blocking(self):
        return self.status == THREAD_STATUS_BLOCKING

    def update_global_thread_id(self):
        QlLinuxThread.LINUX_THREAD_ID = os.getpid()


class QlLinuxX86Thread(QlLinuxThread):
    """docstring for X86Thread"""
    def __init__(self, ql, start_address, exit_point, context = None, set_child_tid_addr = None, thread_id = None):
        super(QlLinuxX86Thread, self).__init__(ql, start_address, exit_point, context, set_child_tid_addr, thread_id)
        self.tls = bytes(b'\x00' * (8 * 3))

    def set_thread_tls(self, tls_addr):
        old_tls = bytes(self.ql.os.gdtm.get_gdt_buf(12, 14 + 1))

        self.ql.os.gdtm.set_gdt_buf(12, 14 + 1, self.tls)

        u_info = self.ql.mem.read(tls_addr, 4 * 4)
        index = self.ql.unpack32s(u_info[0 : 4])
        base = self.ql.unpack32(u_info[4 : 8])
        limit = self.ql.unpack32(u_info[8 : 12])

        if index == -1:
            index = self.ql.os.gdtm.get_free_idx(12)

        if index == -1 or index < 12 or index > 14:
            raise
        else:
            self.ql.os.gdtm.register_gdt_segment(index, base, limit, QL_X86_A_PRESENT | QL_X86_A_DATA | QL_X86_A_DATA_WRITABLE | QL_X86_A_PRIV_3 | QL_X86_A_DIR_CON_BIT, QL_X86_S_GDT | QL_X86_S_PRIV_3)
            self.ql.mem.write(tls_addr, self.ql.pack32(index))

        self.tls = bytes(self.ql.os.gdtm.get_gdt_buf(12, 14 + 1))
        self.ql.os.gdtm.set_gdt_buf(12, 14 + 1, old_tls)
        self.ql.log.debug(f"Set tls to index={hex(index)} base={hex(base)} limit={hex(limit)} fs={hex(self.ql.reg.fs)} gs={hex(self.ql.reg.gs)} gdt_buf={self.tls}")

    def save(self):
        self.save_context()
        self.tls = bytes(self.ql.os.gdtm.get_gdt_buf(12, 14 + 1))
        self.ql.log.debug(f"Saved context. fs={hex(self.ql.reg.fs)} gs={hex(self.ql.reg.gs)} gdt_buf={self.tls}")

    def restore(self):
        self.restore_context()
        self.ql.os.gdtm.set_gdt_buf(12, 14 + 1, self.tls)
        self.ql.log.debug(f"Restored context. fs={hex(self.ql.reg.fs)} gs={hex(self.ql.reg.gs)} gdt_buf={self.tls}")

    def clone(self):
        new_thread = super(QlLinuxX86Thread, self).clone()
        new_thread.tls = self.tls
        return new_thread

class QlLinuxX8664Thread(QlLinuxThread):
    """docstring for X8664Thread"""
    def __init__(self, ql, start_address, exit_point, context = None, set_child_tid_addr = None, thread_id = None):
        super(QlLinuxX8664Thread, self).__init__(ql, start_address, exit_point, context, set_child_tid_addr, thread_id)
        self.tls = 0

    def set_thread_tls(self, tls_addr):
        self.tls = tls_addr
        self.ql.reg.msr(FSMSR, self.tls)
        self.ql.log.debug(f"Set fsbase to {hex(tls_addr)} for {str(self)}")

    # Some notes:
    #     - https://wiki.osdev.org/SWAPGS
    #     - https://stackoverflow.com/questions/11497563/detail-about-msr-gs-base-in-linux-x86-64
    def save(self):
        self.save_context()
        self.tls = self.ql.reg.msr(FSMSR)
        self.ql.log.debug(f"Saved context: fs={hex(self.ql.reg.fsbase)} tls={hex(self.tls)}")

    def restore(self):
        self.restore_context()
        self.set_thread_tls(self.tls)
        self.ql.log.debug(f"Restored context: fs={hex(self.ql.reg.fsbase)} tls={hex(self.tls)}")
    
    def clone(self):
        new_thread = super(QlLinuxX8664Thread, self).clone()
        new_thread.tls = self.tls
        return new_thread

class QlLinuxMIPS32Thread(QlLinuxThread):
    """docstring for QlLinuxMIPS32Thread"""
    def __init__(self, ql, start_address, exit_point, context = None, set_child_tid_addr = None, thread_id = None):
        super(QlLinuxMIPS32Thread, self).__init__(ql, start_address, exit_point, context, set_child_tid_addr, thread_id)
        self.tls = 0


    def set_thread_tls(self, tls_addr):
        self.tls = tls_addr
        CONFIG3_ULR = (1 << 13)
        self.ql.reg.cp0_config3 = CONFIG3_ULR
        self.ql.reg.cp0_userlocal = self.tls
        self.ql.log.debug(f"Set cp0 to {hex(self.ql.reg.cp0_userlocal)}")

    def save(self):
        self.save_context()
        self.tls = self.ql.reg.cp0_userlocal
        self.ql.log.debug(f"Saved context. cp0={hex(self.ql.reg.cp0_userlocal)}") 

    def restore(self):
        self.restore_context()
        self.set_thread_tls(self.tls)
        self.ql.log.debug(f"Restored context. cp0={hex(self.ql.reg.cp0_userlocal)}")

    def clone(self):
        new_thread = super(QlLinuxMIPS32Thread, self).clone()
        new_thread.tls = self.tls
        return new_thread

class QlLinuxARMThread(QlLinuxThread):
    """docstring for QlLinuxARMThread"""
    def __init__(self, ql, start_address, exit_point, context = None, set_child_tid_addr = None, thread_id = None):
        super(QlLinuxARMThread, self).__init__(ql, start_address, exit_point, context, set_child_tid_addr, thread_id)
        self.tls = 0


    def set_thread_tls(self, tls_addr):
        self.tls = tls_addr
        self.ql.reg.c13_c0_3 = self.tls
        self.ql.log.debug(f"Set c13_c0_3 to {hex(self.ql.reg.c13_c0_3)}")

    def save(self):
        self.save_context()
        self.tls = self.ql.reg.c13_c0_3
        self.ql.log.debug(f"Saved context. c13_c0_3={hex(self.ql.reg.c13_c0_3)}")


    def restore(self):
        self.restore_context()
        self.set_thread_tls(self.tls)
        self.ql.log.debug(f"Restored context. c13_c0_3={hex(self.ql.reg.c13_c0_3)}")
    
    def clone(self):
        new_thread = super(QlLinuxARMThread, self).clone()
        new_thread.tls = self.tls
        return new_thread


class QlLinuxARM64Thread(QlLinuxThread):
    """docstring for QlLinuxARM64Thread"""
    def __init__(self, ql, start_address, exit_point, context = None, set_child_tid_addr = None, thread_id = None):
        super(QlLinuxARM64Thread, self).__init__(ql, start_address, exit_point, context, set_child_tid_addr, thread_id)
        self.tls = 0

    def set_thread_tls(self, tls_addr):
        self.tls = tls_addr
        self.ql.reg.tpidr_el0 = self.tls
        self.ql.log.debug(f"Set tpidr_el0 to {hex(self.ql.reg.tpidr_el0)}")

    def save(self):
        self.save_context()
        self.tls = self.ql.reg.tpidr_el0
        self.ql.log.debug(f"Saved context. tpidr_el0={hex(self.ql.reg.tpidr_el0)}")

    def restore(self):
        self.restore_context()
        self.set_thread_tls(self.tls)
        self.ql.log.debug(f"Restored context. tpidr_el0={hex(self.ql.reg.tpidr_el0)}")
    
    def clone(self):
        new_thread = super(QlLinuxARM64Thread, self).clone()
        new_thread.tls = self.tls
        return new_thread

class QlLinuxThreadManagement:
    def __init__(self, ql):
        self.ql = ql
        self.threads = set()
        self.runing_time = 0
        self._main_thread = None
        self._cur_thread = None

    # cur_thread is only guaranteed to be correct in unicorn callbacks context.
    @property
    def cur_thread(self):
        return self._cur_thread

    @cur_thread.setter
    def cur_thread(self, ct):
        self._cur_thread = ct

    @property
    def main_thread(self):
        return self._main_thread
    
    @main_thread.setter
    def main_thread(self, mt):
        self._main_thread = mt

    def stop_thread(self, t):
        t.stop()
        if t in self.threads:
            self.threads.remove(t)
        # Exit the world.
        if t == self.main_thread:
            self.stop()

    def _clear_queued_msg(self):
        try:
            msg_before_main_thread = self.ql._msg_before_main_thread
            for lvl, msg in msg_before_main_thread:
                self.main_thread.log_file_fd.log(lvl, msg)
        except AttributeError:
            pass
    
    def _prepare_lib_patch(self):
        if self.ql.loader.elf_entry != self.ql.loader.entry_point:
            self.main_thread = self.ql.os.thread_class.spawn(self.ql, self.ql.loader.entry_point, self.ql.loader.elf_entry)
            self.cur_thread = self.main_thread
            self._clear_queued_msg()
            gevent.joinall([self.main_thread], raise_error=True)
            if self.ql.reg.arch_pc != self.ql.loader.elf_entry:
                self.ql.log.error(f"{self.cur_thread} Expect {hex(self.ql.loader.elf_entry)} but get {hex(self.ql.reg.arch_pc)} when running loader.")
                raise QlErrorExecutionStop('Dynamic library .init() failed!')
            self.ql.enable_lib_patch()
            self.ql.os.run_function_after_load()
            self.ql.loader.skip_exit_check = False
            self.ql.write_exit_trap()
            return self.main_thread
        return None

    # Stop the world, urge all threads to stop immediately.
    def stop(self):
        self.ql.log.debug("[Thread Manager] Stop the world.")
        self.ql.emu_stop()
        for t in self.threads:
            gevent.kill(t)

    def run(self):
        previous_thread = self._prepare_lib_patch()
        if previous_thread is None:
            self.main_thread = self.ql.os.thread_class.spawn(self.ql, self.ql.loader.elf_entry, self.ql.os.exit_point)
        else:
            self.main_thread = previous_thread.clone()
            self.main_thread.start_address = self.ql.loader.elf_entry
            self.main_thread.exit_point = self.ql.os.exit_point
        self.cur_thread = self.main_thread
        self._clear_queued_msg()
        # If we get exceptions from gevent here, it means a critical bug related to multithread.
        # Please fire an issue if you encounter an exception from gevent.
        gevent.joinall([self.main_thread], raise_error=True)




#!/usr/bin/env python3
# 
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#

# virtual addr :  0x00000000000 -  0x7FFFFFFFFFF
    # 64 bit case no aslr:
        # 0x000000000 - 0x100000000  page zero
        # 0x100000000 - 0xXXXXXXXXX  binary
        # 0xXXXXXXXXX - 0xYYYYYYYYY  dyld
        # 0x7ffcf0000000 - 0x7ffd09a00000  stack 
        # 0x7ffbf0100000 - 0x7ffcf0000000  heap
        # 0x7FFF00000000 - 0x7fffffe00000  shared origen
# kernel space :  0x80000000000 -  0xFFFFFFFFFFF
# other space  :  0x100000000000 - 0x17FFFFFFFFFF


#!/usr/bin/env python3
#
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#

import types

from unicorn import *

from qiling.arch.x86_const import *
from qiling.arch.x86 import *
from qiling.const import *
from qiling.os.os import QlOs
from qiling.os.fncc import QlOsFncc

from .dlls import *
from .const import *
from .utils import *


class QlOsWindows(QlOs, QlOsFncc):
    def __init__(self, ql):
        QlOs.__init__(self, ql)
        QlOsFncc.__init__(self, ql)
        self.ql = ql
        self.PE_RUN = True
        self.last_error = 0
        # variables used inside hooks
        self.user_defined_api = {}
        self.user_defined_api_onenter = {}
        self.user_defined_api_onexit = {}
        self.hooks_variables = {}
        self.syscall_count = {}
        self.argv = self.ql.argv
        self.env = self.ql.env
        self.pid = self.profile.getint("KERNEL","pid")
        self.ql.hook_mem_unmapped(ql_x86_windows_hook_mem_error)
        self.automatize_input = self.profile.getboolean("MISC","automatize_input")
        self.username = self.profile["USER"]["username"]
        self.windir = self.profile["PATH"]["systemdrive"] + self.profile["PATH"]["windir"]
        self.userprofile = self.profile["PATH"]["systemdrive"] + "Users\\" + self.profile["USER"]["username"] + "\\"
        self.load()


    def load(self):
        self.setupGDT()
        # hook win api
        self.ql.hook_code(self.hook_winapi)
    

    def setupGDT(self):
        # setup gdt
        if self.ql.archtype == QL_ARCH.X86:
            self.gdtm = GDTManager(self.ql)
            ql_x86_register_cs(self)
            ql_x86_register_ds_ss_es(self)
            ql_x86_register_fs(self)
            ql_x86_register_gs(self)
        elif self.ql.archtype == QL_ARCH.X8664:
            ql_x8664_set_gs(self.ql)


    def setupComponents(self):
        # handle manager
        self.handle_manager = HandleManager()
        # registry manger
        self.registry_manager = RegistryManager(self.ql)
        # clipboard
        self.clipboard = Clipboard(self.ql.os)
        # fibers
        self.fiber_manager = FiberManager(self.ql)
        # thread manager
        main_thread = QlWindowsThread(self.ql)
        self.thread_manager = QlWindowsThreadManagement(self.ql, main_thread)

        # more handle manager
        new_handle = Handle(obj=main_thread)
        self.handle_manager.append(new_handle)


    # hook WinAPI in PE EMU
    def hook_winapi(self, int, address, size):
        if address in self.ql.loader.import_symbols:
            winapi_name = self.ql.loader.import_symbols[address]['name']
            if winapi_name is None:
                winapi_name = Mapper[self.ql.loader.import_symbols[address]['dll']][self.ql.loader.import_symbols[address]['ordinal']]
            else:
                winapi_name = winapi_name.decode()
            winapi_func = None

            if winapi_name in self.user_defined_api:
                if isinstance(self.user_defined_api[winapi_name], types.FunctionType):
                    winapi_func = self.user_defined_api[winapi_name]
            else:
                try:
                    counter = self.syscall_count.get(winapi_name, 0) + 1
                    self.syscall_count[winapi_name] = counter
                    winapi_func = globals()['hook_' + winapi_name]
                except KeyError:
                    winapi_func = None
            
            if winapi_name in self.user_defined_api_onenter:
                if isinstance(self.user_defined_api_onenter[winapi_name], types.FunctionType):
                    self.winapi_func_onenter = self.user_defined_api_onenter[winapi_name]
            else:
                self.winapi_func_onenter = None

            if winapi_name in self.user_defined_api_onexit:
                if isinstance(self.user_defined_api_onexit[winapi_name], types.FunctionType):
                    self.winapi_func_onexit = self.user_defined_api_onexit[winapi_name]
            else:
                self.winapi_func_onexit = None

            if winapi_func:
                try:
                    winapi_func(self.ql, address, {})
                        
                except Exception as ex:
                    self.ql.log.exception(ex)
                    self.ql.log.info("%s Exception Found" % winapi_name)
                    self.emu_error()
                    raise QlErrorSyscallError("Windows API Implementation Error")
            else:
                self.ql.log.warning("%s is not implemented" % winapi_name)
                if self.ql.debug_stop:
                    raise QlErrorSyscallNotFound("Windows API Implementation Not Found")


    def run(self):
        if self.ql.exit_point is not None:
            self.exit_point = self.ql.exit_point
        
        if  self.ql.entry_point is not None:
            self.ql.loader.entry_point = self.ql.entry_point

        if self.ql.stdin != 0:
            self.stdin = self.ql.stdin
        
        if self.ql.stdout != 0:
            self.stdout = self.ql.stdout
        
        if self.ql.stderr != 0:
            self.stderr = self.ql.stderr
        
        try:
            if self.ql.code:
                self.ql.emu_start(self.ql.loader.entry_point, (self.ql.loader.entry_point + len(self.ql.code)), self.ql.timeout, self.ql.count)
            else:
                self.ql.emu_start(self.ql.loader.entry_point, self.exit_point, self.ql.timeout, self.ql.count)
        except UcError:
            self.emu_error()
            raise

        self.registry_manager.save()
        self.post_report()




#!/usr/bin/env python3
# 
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#


import sys, traceback

class QlErrorBase(Exception):
    def __init__(self, msg):
        super().__init__(self)
        self.msg = msg

    def __str__(self):
        return self.msg

class QlErrorStructConversion(QlErrorBase):
    pass

class QlErrorFileNotFound(QlErrorBase):
    pass

class QlErrorFileType(QlErrorBase):
    pass

class QlErrorOsType(QlErrorBase):
    pass

class QlErrorOutput(QlErrorBase):
    pass

class QlErrorArch(QlErrorBase):
    pass

class QlErrorRuntype(QlErrorBase):
    pass

class QlErrorJsonDecode(QlErrorBase):
    pass

class QlErrorNotImplemented(QlErrorBase):
    pass

class QlErrorELFFormat(QlErrorBase):
    pass

class QlErrorMACHOFormat(QlErrorBase):
    pass

class QlErrorModuleFunctionNotFound(QlErrorBase):
    pass

class QlErrorModuleNotFound(QlErrorBase):
    pass

class QlErrorExecutionStop(QlErrorBase):
    pass

class QlErrorSyscallError(QlErrorBase):
    pass

class QlErrorSyscallNotFound(QlErrorBase):
    pass

class QlErrorCoreHook(QlErrorBase):
    pass

class QlOutOfMemory(QlErrorBase):
    pass

class QlMemoryMappedError(QlErrorBase):
    pass

class QlGDTError(QlErrorBase):
    pass

class QlSyscallError(QlErrorBase):
    def __init__(self, errno, msg):
        super(QlSyscallError, self).__init__(msg)
        self.errno = errno
    
    def __str__(self):
        return "[ Errno " + str(self.errno) + "] " + self.msg

def QlPrintException(msg):
    _, _, exc_traceback = sys.exc_info()
    print(msg + repr(traceback.format_tb(exc_traceback, limit=1) ) )
    


#!/usr/bin/env python3
# 
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#

import os, string, sys

from heapq import heappush, heappop

from elftools.elf.elffile import ELFFile
from elftools.elf.relocation import RelocationSection
from elftools.elf.sections import SymbolTableSection
from elftools.elf.descriptions import describe_reloc_type

from qiling.const import *

from qiling.exception import *
from .loader import QlLoader
from qiling.os.linux.function_hook import FunctionHook
from qiling.os.linux.syscall_nums import *
from qiling.os.linux.kernel_api.hook import *

AT_NULL = 0
AT_IGNORE = 1
AT_EXECFD = 2
AT_PHDR = 3
AT_PHENT = 4
AT_PHNUM = 5
AT_PAGESZ = 6
AT_BASE = 7
AT_FLAGS = 8
AT_ENTRY = 9
AT_NOTELF = 10
AT_UID = 11
AT_EUID = 12
AT_GID = 13
AT_EGID = 14
AT_PLATFORM = 15
AT_HWCAP = 16
AT_CLKTCK = 17
AT_SECURE = 23
AT_BASE_PLATFORM = 24
AT_RANDOM = 25
AT_HWCAP2 = 26
AT_EXECFN = 31

FILE_DES = []

# start area memory for API hooking
# we will reserve 0x1000 bytes for this (which contains multiple slots of 4/8 bytes, each for one api)
API_HOOK_MEM = 0x1000000

# SYSCALL_MEM = 0xffff880000000000
# memory for syscall table
SYSCALL_MEM = API_HOOK_MEM + 0x1000


class ELFParse():
    def __init__(self, path, ql):
        self.path = os.path.abspath(path)
        self.ql = ql

        self.f = open(path, "rb")
        elfdata = self.f.read()
        self.elffile = ELFFile(self.f)

        self.elfdata = elfdata.ljust(52, b'\x00')

        if self.elffile.e_ident_raw[: 4] != b'\x7fELF':
            raise QlErrorELFFormat("ERROR: NOT a ELF")

        self.elfhead = self.parse_header()
        if self.elfhead['e_type'] == "ET_REL":  # kernel driver
            self.is_driver = True
        else:
            self.is_driver = False

    def getelfdata(self, offest, size):
        return self.elfdata[offest: offest + size]

    def parse_header(self):
        return dict(self.elffile.header)

    def parse_sections(self):
        return self.elffile.iter_sections()

    def parse_segments(self):
        return self.elffile.iter_segments()


class QlLoaderELF(QlLoader, ELFParse):
    def __init__(self, ql):
        super(QlLoaderELF, self).__init__(ql)
        self.ql = ql

    def run(self):
        if self.ql.code:
            self.ql.mem.map(self.ql.os.entry_point, self.ql.os.code_ram_size, info="[shellcode_stack]")
            self.ql.os.entry_point = (self.ql.os.entry_point + 0x200000 - 0x1000)
            self.ql.mem.write(self.ql.os.entry_point, self.ql.code)
            self.ql.reg.arch_sp = self.ql.os.entry_point
            return

        if self.ql.archbit == 32:
            stack_address = int(self.ql.os.profile.get("OS32", "stack_address"), 16)
            stack_size = int(self.ql.os.profile.get("OS32", "stack_size"), 16)
        elif self.ql.archbit == 64:
            stack_address = int(self.ql.os.profile.get("OS64", "stack_address"), 16)
            stack_size = int(self.ql.os.profile.get("OS64", "stack_size"), 16)

        self.path = self.ql.path
        ELFParse.__init__(self, self.path, self.ql)
        self.interp_address = 0
        self.mmap_address = 0
        self.ql.mem.map(stack_address, stack_size, info="[stack]")

        if self.ql.ostype == QL_OS.FREEBSD:
            init_rbp = stack_address + 0x40
            init_rdi = stack_address
            self.ql.reg.rbp = init_rbp
            self.ql.reg.rdi = init_rdi
            self.ql.reg.r14 = init_rdi

        if not self.is_driver:
            self.load_with_ld(stack_address + stack_size, argv=self.argv, env=self.env)
        else:

            # Linux kernel driver
            if self.load_driver(self.ql, stack_address + stack_size):
                raise QlErrorFileType("Unsupported FileType")
            # hook Linux kernel api
            self.ql.hook_code(hook_kernel_api)

        self.ql.reg.arch_sp = self.stack_address
        self.ql.os.stack_address = self.stack_address

    # Copy strings to stack.
    def copy_str(self, addr, strs):
        l_addr = []
        s_addr = addr
        for s in strs:
            bs = s.encode("utf-8") + b"\x00" if not isinstance(s, bytes) else s
            s_addr = s_addr - len(bs)
            self.ql.mem.write(s_addr, bs)
            l_addr.append(s_addr)
        return l_addr, s_addr

    def alignment(self, val):
        if self.ql.archbit == 64:
            return (val // 8) * 8
        elif self.ql.archbit == 32:
            return (val // 4) * 4

    def NEW_AUX_ENT(self, key, val):
        return self.ql.pack(int(key)) + self.ql.pack(int(val))

    def NullStr(self, s):
        return s[: s.find(b'\x00')]

    def pcalc(self, length, align):
        tmp = length // align
        if length % align:
            tmp = tmp + 1
        return tmp * align

    def load_with_ld(self, stack_addr, load_address=-1, argv=[], env={}):
        pagesize = 0x1000
        _mem_e = 0

        if load_address <= 0:
            if self.ql.archbit == 64:
                load_address = int(self.ql.os.profile.get("OS64", "load_address"), 16)
            else:
                load_address = int(self.ql.os.profile.get("OS32", "load_address"), 16)

        elfhead = super().parse_header()

        # Correct the load_address if needed
        if elfhead['e_type'] == 'ET_EXEC':
            load_address = 0
        elif elfhead['e_type'] != 'ET_DYN':
            self.ql.log.debug("Some error in head e_type: %i!", elfhead['e_type'])
            return -1

        # We need to sort the memory segments first, sometimes they are unordered
        loadheap = []
        for entry in super().parse_segments():
            if entry['p_type'] == 'PT_LOAD' or entry['p_type'] == 'PT_INTERP':
                paddr = entry['p_vaddr']
                heappush(loadheap, (paddr, entry))
        loaddb = [dict(heappop(loadheap)[1].header) for i in range(len(loadheap))]

        # Determine the range of memory space opened up
        mem_start = -1
        mem_end = -1
        interp_path = ''
        for entry in loaddb:
            if entry['p_type'] == 'PT_LOAD':
                if mem_start > entry['p_vaddr'] or mem_start == -1:
                    mem_start = entry['p_vaddr']
                if mem_end < entry['p_vaddr'] + entry['p_memsz'] or mem_end == -1:
                    mem_end = entry['p_vaddr'] + entry['p_memsz']
            if entry['p_type'] == 'PT_INTERP':
                interp_path = self.NullStr(super().getelfdata(entry['p_offset'], entry['p_filesz']))

        mem_start = int(mem_start // 0x1000) * 0x1000
        mem_end = int(mem_end // 0x1000 + 1) * 0x1000

        # Now we calculate the segments based on page alignment
        _load_segments = {}
        _last_start = 0
        _last_end = 0
        _last_perm = 0
        for entry in loaddb:
            if entry['p_type'] == 'PT_LOAD':
                _mem_start = ((load_address + entry["p_vaddr"]) // pagesize) * pagesize
                _mem_len = entry['p_memsz']
                _mem_end = self.pcalc(load_address + entry["p_vaddr"] + _mem_len, pagesize)
                _perms = int(bin(entry["p_flags"])[:1:-1], 2)  # reverse bits for perms mapping
                if _last_end < _mem_start:
                    _load_segments[_mem_start] = _mem_end, _perms
                    _last_start = _mem_start
                elif _perms == _last_perm:
                    _load_segments[_last_start] = _mem_end, _perms
                elif _last_end == _mem_start:
                    _load_segments[_mem_start] = _mem_end, _perms
                    _last_start = _mem_start
                elif _mem_start<_last_end:
                    _load_segments[_last_start]=_mem_end,_perms
                _last_end = _mem_end
                _last_perm = _perms


        # Let's map the memory first
        _highestmapped_e = 0
        for segment in _load_segments:
            _mem_s = segment
            _mem_e = _load_segments[segment][0]
            _perms = _load_segments[segment][1] & 0xFF
            try:
                self.ql.mem.map(_mem_s, _mem_e - _mem_s, perms=_perms, info=self.path)
                if _mem_e > _highestmapped_e:
                    _highestmapped_e = _mem_e
                self.ql.log.debug("load 0x%x - 0x%x" % (_mem_s, _mem_e))
            except Exception as e:
                self.ql.log.debug("load 0x%x - 0x%x => %s" % (_mem_s, _mem_e, str(e)))
                continue

        # Now we write the segment data to the memory
        for entry in loaddb:
            if entry['p_type'] == 'PT_LOAD' and entry['p_filesz'] > 0:
                try:
                    _mem_s = load_address + entry["p_vaddr"]
                    data = super().getelfdata(entry['p_offset'], entry['p_filesz'])
                    self.ql.mem.write(_mem_s, data)
                except Exception as e:
                    self.ql.log.debug("segment data 0x%x - Length 0x%x => %s" % (_mem_s, len(data), str(e)))
                    continue

        loaded_mem_end = load_address + mem_end
        if loaded_mem_end > _mem_e:
            self.ql.mem.map(_mem_e, loaded_mem_end - _mem_e, info=self.path)
            self.ql.log.debug("load 0x%x - 0x%x" % (
            _mem_e, loaded_mem_end))  # make sure we map all PT_LOAD tagged area

        entry_point = elfhead['e_entry'] + load_address
        self.ql.os.elf_mem_start = mem_start
        self.ql.log.debug("mem_start: 0x%x mem_end: 0x%x" % (mem_start, mem_end))

        self.brk_address = mem_end + load_address + 0x2000

        # Load interpreter if there is an interpreter

        if interp_path != '':
            interp_path = str(interp_path, 'utf-8', errors="ignore")

            interp = ELFParse(self.ql.rootfs + interp_path, self.ql)
            interphead = interp.parse_header()
            self.ql.log.debug("interp is : %s" % (self.ql.rootfs + interp_path))

            interp_mem_size = -1
            for i in interp.parse_segments():
                i = dict(i.header)
                if i['p_type'] == 'PT_LOAD':
                    if interp_mem_size < i['p_vaddr'] + i['p_memsz'] or interp_mem_size == -1:
                        interp_mem_size = i['p_vaddr'] + i['p_memsz']

            interp_mem_size = (interp_mem_size // 0x1000 + 1) * 0x1000
            self.ql.log.debug("interp_mem_size is : 0x%x" % int(interp_mem_size))

            if self.ql.archbit == 64:
                self.interp_address = int(self.ql.os.profile.get("OS64", "interp_address"), 16)
            elif self.ql.archbit == 32:
                self.interp_address = int(self.ql.os.profile.get("OS32", "interp_address"), 16)

            self.ql.log.debug("interp_address is : 0x%x" % (self.interp_address))
            self.ql.mem.map(self.interp_address, int(interp_mem_size),
                            info=os.path.abspath(self.ql.rootfs + interp_path))

            for i in interp.parse_segments():
                # i =dict(i.header)
                if i['p_type'] == 'PT_LOAD':
                    self.ql.mem.write(self.interp_address + i['p_vaddr'],
                                      interp.getelfdata(i['p_offset'], i['p_filesz']))
            entry_point = interphead['e_entry'] + self.interp_address

        # Set MMAP addr
        if self.ql.archbit == 64:
            self.mmap_address = int(self.ql.os.profile.get("OS64", "mmap_address"), 16)
        else:
            self.mmap_address = int(self.ql.os.profile.get("OS32", "mmap_address"), 16)

        self.ql.log.debug("mmap_address is : 0x%x" % (self.mmap_address))

        # Set elf table
        elf_table = b''
        new_stack = stack_addr

        # Set argc
        elf_table += self.ql.pack(len(argv))

        # Set argv
        if len(argv) != 0:
            argv_addr, new_stack = self.copy_str(stack_addr, argv)
            elf_table += b''.join([self.ql.pack(_) for _ in argv_addr])

        elf_table += self.ql.pack(0)

        # Set env
        if len(env) != 0:
            env_addr, new_stack = self.copy_str(new_stack, [key + '=' + value for key, value in env.items()])
            elf_table += b''.join([self.ql.pack(_) for _ in env_addr])

        elf_table += self.ql.pack(0)

        new_stack = self.alignment(new_stack)
        randstr = 'a' * 0x10
        cpustr = 'i686'
        (addr, new_stack) = self.copy_str(new_stack, [randstr, cpustr])
        new_stack = self.alignment(new_stack)

        # Set AUX
        self.elf_phdr = (load_address + elfhead['e_phoff'])
        self.elf_phent = (elfhead['e_phentsize'])
        self.elf_phnum = (elfhead['e_phnum'])
        self.elf_pagesz = 0x1000
        self.elf_guid = self.ql.os.uid
        self.elf_flags = 0
        self.elf_entry = (load_address + elfhead['e_entry'])
        self.randstraddr = addr[0]
        self.cpustraddr = addr[1]
        if self.ql.archbit == 64:
            self.elf_hwcap = 0x078bfbfd
        elif self.ql.archbit == 32:
            self.elf_hwcap = 0x1fb8d7
            if self.ql.archendian == QL_ENDIAN.EB:
                self.elf_hwcap = 0xd7b81f

        elf_table += self.NEW_AUX_ENT(AT_PHDR, self.elf_phdr + mem_start)
        elf_table += self.NEW_AUX_ENT(AT_PHENT, self.elf_phent)
        elf_table += self.NEW_AUX_ENT(AT_PHNUM, self.elf_phnum)
        elf_table += self.NEW_AUX_ENT(AT_PAGESZ, self.elf_pagesz)
        elf_table += self.NEW_AUX_ENT(AT_BASE, self.interp_address)
        elf_table += self.NEW_AUX_ENT(AT_FLAGS, self.elf_flags)
        elf_table += self.NEW_AUX_ENT(AT_ENTRY, self.elf_entry)
        elf_table += self.NEW_AUX_ENT(AT_UID, self.elf_guid)
        elf_table += self.NEW_AUX_ENT(AT_EUID, self.elf_guid)
        elf_table += self.NEW_AUX_ENT(AT_GID, self.elf_guid)
        elf_table += self.NEW_AUX_ENT(AT_EGID, self.elf_guid)
        elf_table += self.NEW_AUX_ENT(AT_HWCAP, self.elf_hwcap)
        elf_table += self.NEW_AUX_ENT(AT_CLKTCK, 100)
        elf_table += self.NEW_AUX_ENT(AT_RANDOM, self.randstraddr)
        elf_table += self.NEW_AUX_ENT(AT_PLATFORM, self.cpustraddr)
        elf_table += self.NEW_AUX_ENT(AT_SECURE, 0)
        elf_table += self.NEW_AUX_ENT(AT_NULL, 0)
        elf_table += b'\x00' * (0x10 - (new_stack - len(elf_table)) & 0xf)

        self.ql.mem.write(new_stack - len(elf_table), elf_table)
        new_stack = new_stack - len(elf_table)

        # self.ql.reg.write(UC_X86_REG_RDI, new_stack + 8)

        # for i in range(120):
        #     buf = self.ql.mem.read(new_stack + i * 0x8, 8)
        #     self.ql.log.info("0x%08x : 0x%08x " % (new_stack + i * 0x4, self.ql.unpack64(buf)) + ' '.join(['%02x' % i for i in buf]) + '  ' + ''.join([chr(i) if i in string.printable[ : -5].encode('ascii') else '.' for i in buf]))

        self.ql.os.entry_point = self.entry_point = entry_point
        self.ql.os.elf_entry = self.elf_entry = load_address + elfhead['e_entry']
        self.stack_address = new_stack
        self.load_address = load_address
        self.images.append(self.coverage_image(load_address, load_address + mem_end, self.path))
        self.ql.os.function_hook = FunctionHook(self.ql, self.elf_phdr + mem_start, self.elf_phnum, self.elf_phent,
                                                load_address, load_address + mem_end)
        self.init_sp = self.ql.reg.arch_sp

        # If there is a loader, we ignore exit
        self.skip_exit_check = self.elf_entry != self.entry_point

        # map vsyscall section for some specific needs
        if self.ql.archtype == QL_ARCH.X8664 and self.ql.ostype == QL_OS.LINUX:
            _vsyscall_addr = int(self.ql.os.profile.get("OS64", "vsyscall_address"), 16)
            _vsyscall_size = int(self.ql.os.profile.get("OS64", "vsyscall_size"), 16)

            if not self.ql.mem.is_mapped(_vsyscall_addr, _vsyscall_size):
                # initialize with \xcc then insert syscall entry
                # each syscall should be 1KiB(0x400 bytes) away
                self.ql.mem.map(_vsyscall_addr, _vsyscall_size, info="[vsyscall]")
                self.ql.mem.write(_vsyscall_addr, _vsyscall_size * b'\xcc')
                assembler = self.ql.create_assembler()

                def _compile(asm):
                    bs, _ = assembler.asm(asm)
                    return bytes(bs)

                _vsyscall_entry_asm = ["mov rax, 0x60;",  # syscall gettimeofday
                                       "mov rax, 0xc9;",  # syscall time
                                       "mov rax, 0x135;",  # syscall getcpu
                                       ]

                for idx, val in enumerate(_vsyscall_entry_asm):
                    self.ql.mem.write(_vsyscall_addr + idx * 0x400, _compile(val + "; syscall; ret"))

    # get file offset of init module function
    def lkm_get_init(self, ql):
        elffile = ELFFile(open(ql.path, 'rb'))
        symbol_tables = [s for s in elffile.iter_sections() if isinstance(s, SymbolTableSection)]
        for section in symbol_tables:
            for nsym, symbol in enumerate(section.iter_symbols()):
                if symbol.name == 'init_module':
                    addr = symbol.entry.st_value + elffile.get_section(symbol['st_shndx'])['sh_offset']
                    ql.log.info("init_module = 0x%x" % addr)
                    return addr

        # not found. FIXME: report error on invalid module??
        ql.log.warning("invalid module? symbol init_module not found")
        return -1

    def lkm_dynlinker(self, ql, mem_start):
        def get_symbol(elffile, name):
            section = elffile.get_section_by_name('.symtab')
            for symbol in section.iter_symbols():
                if symbol.name == name:
                    return symbol
            return None

        elffile = ELFFile(open(ql.path, 'rb'))

        all_symbols = []
        self.ql.os.hook_addr = API_HOOK_MEM
        # map address to symbol name
        self.import_symbols = {}
        # reverse dictionary to map symbol name -> address
        rev_reloc_symbols = {}

        # dump_mem("XX Original code at 15a1 = ", ql.mem.read(0x15a1, 8))
        _sections = list(elffile.iter_sections())
        for section in _sections:
            # only care about reloc section
            if not isinstance(section, RelocationSection):
                continue

            # ignore reloc for module section
            if section.name == ".rela.gnu.linkonce.this_module":
                continue

            dest_sec_idx = section.header.get('sh_info', None)
            if dest_sec_idx is not None and dest_sec_idx < len(_sections):
                dest_sec = _sections[dest_sec_idx]
                if dest_sec.header['sh_flags'] & 2 == 0:
                    # The target section is not loaded into memory, so just continue
                    continue

            # The symbol table section pointed to in sh_link
            symtable = elffile.get_section(section['sh_link'])
            for rel in section.iter_relocations():
                if rel['r_info_sym'] == 0:
                    continue

                symbol = symtable.get_symbol(rel['r_info_sym'])

                # Some symbols have zero 'st_name', so instead what's used is
                # the name of the section they point at.
                if symbol['st_name'] == 0:
                    symsec = elffile.get_section(symbol['st_shndx'])  # save sh_addr of this section
                    symbol_name = symsec.name
                    sym_offset = symsec['sh_offset']
                    # we need to do reverse lookup from symbol to address
                    rev_reloc_symbols[symbol_name] = sym_offset + mem_start
                else:
                    symbol_name = symbol.name
                    # get info about related section to be patched
                    info_section = elffile.get_section(section['sh_info'])
                    sym_offset = info_section['sh_offset']

                    if not symbol_name in all_symbols:
                        _symbol = get_symbol(elffile, symbol_name)
                        if _symbol['st_shndx'] == 'SHN_UNDEF':
                            # external symbol
                            # only save symbols of APIs
                            all_symbols.append(symbol_name)
                            # we need to lookup from address to symbol, so we can find the right callback
                            # for sys_xxx handler for syscall, the address must be aligned to 8
                            if symbol_name.startswith('sys_'):
                                if self.ql.os.hook_addr % self.ql.pointersize != 0:
                                    self.ql.os.hook_addr = (int(
                                        self.ql.os.hook_addr / self.ql.pointersize) + 1) * self.ql.pointersize
                                    # print("hook_addr = %x" %self.ql.os.hook_addr)
                            self.import_symbols[self.ql.os.hook_addr] = symbol_name
                            # ql.log.info(":: Demigod is hooking %s(), at slot %x" %(symbol_name, self.ql.os.hook_addr))

                            if symbol_name == "page_offset_base":
                                # FIXME: this is for rootkit to scan for syscall table from page_offset_base
                                # write address of syscall table to this slot,
                                # so syscall scanner can quickly find it
                                ql.mem.write(self.ql.os.hook_addr, self.ql.pack(SYSCALL_MEM))

                            # we also need to do reverse lookup from symbol to address
                            rev_reloc_symbols[symbol_name] = self.ql.os.hook_addr
                            sym_offset = self.ql.os.hook_addr - mem_start
                            self.ql.os.hook_addr += self.ql.pointersize
                        else:
                            # local symbol
                            all_symbols.append(symbol_name)
                            _section = elffile.get_section(_symbol['st_shndx'])
                            rev_reloc_symbols[symbol_name] = _section['sh_offset'] + _symbol['st_value'] + mem_start
                            # ql.log.info(":: Add reverse lookup for %s to %x (%x, %x)" %(symbol_name, rev_reloc_symbols[symbol_name], _section['sh_offset'], _symbol['st_value']))
                            # ql.log.info(":: Add reverse lookup for %s to %x" %(symbol_name, rev_reloc_symbols[symbol_name]))
                    else:
                        sym_offset = rev_reloc_symbols[symbol_name] - mem_start

                # ql.log.info("Relocating symbol %s -> 0x%x" %(symbol_name, rev_reloc_symbols[symbol_name]))

                loc = elffile.get_section(section['sh_info'])['sh_offset'] + rel['r_offset']
                loc += mem_start

                if describe_reloc_type(rel['r_info_type'], elffile) == 'R_X86_64_32S':
                    # patch this reloc
                    if rel['r_addend']:
                        val = sym_offset + rel['r_addend']
                        val += mem_start
                        # ql.log.info('R_X86_64_32S %s: [0x%x] = 0x%x' %(symbol_name, loc, val & 0xFFFFFFFF))
                        ql.mem.write(loc, ql.pack32(val & 0xFFFFFFFF))
                    else:
                        # print("sym_offset = %x, rel = %x" %(sym_offset, rel['r_addend']))
                        # ql.log.info('R_X86_64_32S %s: [0x%x] = 0x%x' %(symbol_name, loc, rev_reloc_symbols[symbol_name] & 0xFFFFFFFF))
                        ql.mem.write(loc, ql.pack32(rev_reloc_symbols[symbol_name] & 0xFFFFFFFF))

                elif describe_reloc_type(rel['r_info_type'], elffile) == 'R_X86_64_64':
                    # patch this function?
                    val = sym_offset + rel['r_addend']
                    val += 0x2000000  # init_module position: FIXME
                    # finally patch this reloc
                    # ql.log.info('R_X86_64_64 %s: [0x%x] = 0x%x' %(symbol_name, loc, val))
                    ql.mem.write(loc, ql.pack64(val))

                elif describe_reloc_type(rel['r_info_type'], elffile) == 'R_X86_64_PC32':
                    # patch branch address: X86 case
                    val = rel['r_addend'] - loc
                    val += rev_reloc_symbols[symbol_name]
                    # finally patch this reloc
                    # ql.log.info('R_X86_64_PC32 %s: [0x%x] = 0x%x' %(symbol_name, loc, val & 0xFFFFFFFF))
                    ql.mem.write(loc, ql.pack32(val & 0xFFFFFFFF))

                elif describe_reloc_type(rel['r_info_type'], elffile) == 'R_386_PC32':
                    val = ql.unpack(ql.mem.read(loc, 4))
                    val = rev_reloc_symbols[symbol_name] + val - loc
                    ql.mem.write(loc, ql.pack32(val & 0xFFFFFFFF))

                elif describe_reloc_type(rel['r_info_type'], elffile) in ('R_386_32', 'R_MIPS_32'):
                    val = ql.unpack(ql.mem.read(loc, 4))
                    val = rev_reloc_symbols[symbol_name] + val
                    ql.mem.write(loc, ql.pack32(val & 0xFFFFFFFF))

                elif describe_reloc_type(rel['r_info_type'], elffile) == 'R_MIPS_HI16':
                    # actual relocation is done in R_MIPS_LO16
                    prev_mips_hi16_loc = loc

                elif describe_reloc_type(rel['r_info_type'], elffile) == 'R_MIPS_LO16':
                    val = ql.unpack16(ql.mem.read(prev_mips_hi16_loc + 2, 2)) << 16 | ql.unpack16(ql.mem.read(loc + 2, 2))
                    val = rev_reloc_symbols[symbol_name] + val
                    # *(word)(mips_lo16_loc + 2) is treated as signed
                    if (val & 0xFFFF) >= 0x8000:
                        val += (1 << 16)

                    ql.mem.write(prev_mips_hi16_loc + 2, ql.pack16(val >> 16))
                    ql.mem.write(loc + 2, ql.pack16(val & 0xFFFF))

                else:
                    raise QlErrorNotImplemented("Relocation type %s not implemented" % describe_reloc_type(rel['r_info_type'], elffile))

        return rev_reloc_symbols

    def load_driver(self, ql, stack_addr, loadbase=0):
        elfhead = super().parse_header()
        elfdata_mapping = self.get_elfdata_mapping()

        # Determine the range of memory space opened up
        mem_start = -1
        mem_end = -1

        # for i in super().parse_program_header(ql):
        #     if i['p_type'] == PT_LOAD:
        #         if mem_start > i['p_vaddr'] or mem_start == -1:
        #             mem_start = i['p_vaddr']
        #         if mem_end < i['p_vaddr'] + i['p_memsz'] or mem_end == -1:
        #             mem_end = i['p_vaddr'] + i['p_memsz']

        # mem_start = int(mem_start // 0x1000) * 0x1000
        # mem_end = int(mem_end // 0x1000 + 1) * 0x1000

        # FIXME
        mem_start = 0x1000
        mem_end = mem_start + int(len(elfdata_mapping) / 0x1000 + 1) * 0x1000

        # map some memory to intercept external functions of Linux kernel
        ql.mem.map(API_HOOK_MEM, 0x1000, info="[api_mem]")

        ql.log.info("loadbase: %x, mem_start: %x, mem_end: %x" % (loadbase, mem_start, mem_end))
        ql.mem.map(loadbase + mem_start, mem_end - mem_start, info=ql.path)
        ql.mem.write(loadbase + mem_start, elfdata_mapping)

        entry_point = self.lkm_get_init(ql) + loadbase + mem_start
        ql.brk_address = mem_end + loadbase

        # Set MMAP addr
        if self.ql.archbit == 64:
            self.mmap_address = int(self.ql.os.profile.get("OS64", "mmap_address"), 16)
        else:
            self.mmap_address = int(self.ql.os.profile.get("OS32", "mmap_address"), 16)

        ql.log.debug("mmap_address is : 0x%x" % (self.mmap_address))

        new_stack = stack_addr
        new_stack = self.alignment(new_stack)

        # self.ql.os.elf_entry = self.elf_entry = loadbase + elfhead['e_entry']
        self.ql.os.entry_point = self.entry_point = entry_point
        self.elf_entry = self.ql.os.elf_entry = self.ql.os.entry_point

        self.stack_address = new_stack
        self.load_address = loadbase

        rev_reloc_symbols = self.lkm_dynlinker(ql, mem_start + loadbase)

        # remember address of syscall table, so external tools can access to it
        ql.os.syscall_addr = SYSCALL_MEM
        # setup syscall table
        ql.mem.map(SYSCALL_MEM, 0x1000, info="[syscall_mem]")
        # zero out syscall table memory
        ql.mem.write(SYSCALL_MEM, b'\x00' * 0x1000)

        # print("sys_close = %x" %rev_reloc_symbols['sys_close'])
        # print(rev_reloc_symbols.keys())
        for sc in rev_reloc_symbols.keys():
            if sc != 'sys_call_table' and sc.startswith('sys_'):
                tmp_sc = sc.replace("sys_", "NR_")
                if tmp_sc in globals():
                    syscall_id = globals()[tmp_sc]
                    ql.log.debug("Writing syscall %s to [0x%x]" % (sc, SYSCALL_MEM + ql.pointersize * syscall_id))
                    ql.mem.write(SYSCALL_MEM + ql.pointersize * syscall_id, ql.pack(rev_reloc_symbols[sc]))

        # write syscall addresses into syscall table
        # ql.mem.write(SYSCALL_MEM + 0, struct.pack("<Q", hook_sys_read))
        ql.mem.write(SYSCALL_MEM + 0, ql.pack(self.ql.os.hook_addr))
        # ql.mem.write(SYSCALL_MEM + 1  * 8, struct.pack("<Q", hook_sys_write))
        ql.mem.write(SYSCALL_MEM + 1 * ql.pointersize, ql.pack(self.ql.os.hook_addr + 1 * ql.pointersize))
        # ql.mem.write(SYSCALL_MEM + 2  * 8, struct.pack("<Q", hook_sys_open))
        ql.mem.write(SYSCALL_MEM + 2 * ql.pointersize, ql.pack(self.ql.os.hook_addr + 2 * ql.pointersize))

        # setup hooks for read/write/open syscalls
        self.import_symbols[self.ql.os.hook_addr] = hook_sys_read
        self.import_symbols[self.ql.os.hook_addr + 1 * ql.pointersize] = hook_sys_write
        self.import_symbols[self.ql.os.hook_addr + 2 * ql.pointersize] = hook_sys_open

    def get_elfdata_mapping(self):
        elfdata_mapping = bytearray()
        elfdata_mapping.extend(self.getelfdata(0, self.elfhead['e_ehsize']))    #elf header

        for section in self.parse_sections():
            if section.header['sh_flags'] & 2:      # alloc flag
                sh_offset = section.header['sh_offset']
                sh_size = section.header['sh_size']

                # align section addr
                elfdata_len = len(elfdata_mapping)
                if elfdata_len < sh_offset:
                    elfdata_mapping.extend(b'\x00' * (sh_offset - elfdata_len))

                if section.header['sh_type'] == 'SHT_NOBITS':
                    elfdata_mapping.extend(b'\x00' * sh_size)
                else:
                    elfdata_mapping.extend(self.getelfdata(sh_offset, sh_size))

        return bytes(elfdata_mapping)



#!/usr/bin/env python3
# 
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#

from unicorn import *
from unicorn.mips_const import *

from qiling.const import *
from .arch import QlArch
from .mips_const import *


class QlArchMIPS(QlArch):
    def __init__(self, ql):
        super(QlArchMIPS, self).__init__(ql)

        register_mappings = [
            reg_map, reg_map_afpr128
        ]

        for reg_maper in register_mappings:
            self.ql.reg.expand_mapping(reg_maper)        

        self.ql.reg.create_reverse_mapping()

        self.ql.reg.register_sp(reg_map["sp"])
        self.ql.reg.register_pc(reg_map["pc"])

    def stack_push(self, value):
        self.ql.reg.sp -= 4
        self.ql.mem.write(self.ql.reg.sp, self.ql.pack32(value))
        return self.ql.reg.sp


    def stack_pop(self):
        data = data = self.ql.unpack32(self.ql.mem.read(self.ql.reg.sp, 4))
        self.ql.reg.sp += 4
        return data
        

    def stack_read(self, offset):
        return self.ql.unpack32(self.ql.mem.read(self.ql.reg.sp + offset, 4))


    def stack_write(self, offset, data):
        return self.ql.mem.write(self.ql.reg.sp + offset, self.ql.pack32(data))

    # get initialized unicorn engine
    def get_init_uc(self):
        if self.ql.archtype== QL_ARCH.MIPS:
            if self.ql.archendian == QL_ENDIAN.EB:
                uc = Uc(UC_ARCH_MIPS, UC_MODE_MIPS32 + UC_MODE_BIG_ENDIAN)
            else:
                uc = Uc(UC_ARCH_MIPS, UC_MODE_MIPS32 + UC_MODE_LITTLE_ENDIAN)
        return uc


#!/usr/bin/env python3
# 
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#

from qiling.const import *
from qiling.os.const import *
from .fncc import *
from .utils import init_struct
from .const import *
from .ProcessorBind import *
from .UefiBaseType import *
from .UefiMultiPhase import *

class EFI_GCD_MEMORY_TYPE(ENUM):
	_members_ = [
		'EfiGcdMemoryTypeNonExistent',
		'EfiGcdMemoryTypeReserved',
		'EfiGcdMemoryTypeSystemMemory',
		'EfiGcdMemoryTypeMemoryMappedIo',
		'EfiGcdMemoryTypePersistent',
		'EfiGcdMemoryTypePersistentMemory',
		'EfiGcdMemoryTypeMoreReliable',
		'EfiGcdMemoryTypeMaximum'
	]

class EFI_GCD_MEMORY_SPACE_DESCRIPTOR(STRUCT):
	_fields_ = [
		('BaseAddress',		EFI_PHYSICAL_ADDRESS),
		('Length',			UINT64),
		('Capabilities',	UINT64),
		('Attributes',		UINT64),
		('GcdMemoryType',	EFI_GCD_MEMORY_TYPE),
		('PADDING_0',		UINT8 * 4),
		('ImageHandle',		EFI_HANDLE),
		('DeviceHandle',	EFI_HANDLE)
	]

class EFI_GCD_IO_TYPE(ENUM):
	_members_ = [
		'EfiGcdIoTypeNonExistent',
		'EfiGcdIoTypeReserved',
		'EfiGcdIoTypeIo',
		'EfiGcdIoTypeMaximum'
	]

class EFI_GCD_IO_SPACE_DESCRIPTOR(STRUCT):
	_fields_ = [
		('BaseAddress',		EFI_PHYSICAL_ADDRESS),
		('Length',			UINT64),
		('GcdIoType',		EFI_GCD_IO_TYPE),
		('PADDING_0',		UINT8 * 4),
		('ImageHandle',		EFI_HANDLE),
		('DeviceHandle',	EFI_HANDLE)
	]

class EFI_GCD_ALLOCATE_TYPE(ENUM):
	_members_ = [
		'EfiGcdAllocateAnySearchBottomUp',
		'EfiGcdAllocateMaxAddressSearchBottomUp',
		'EfiGcdAllocateAddress',
		'EfiGcdAllocateAnySearchTopDown',
		'EfiGcdAllocateMaxAddressSearchTopDown',
		'EfiGcdMaxAllocateType'
	]

EFI_ADD_MEMORY_SPACE				= FUNCPTR(EFI_STATUS, EFI_GCD_MEMORY_TYPE, EFI_PHYSICAL_ADDRESS, UINT64, UINT64)
EFI_ALLOCATE_MEMORY_SPACE			= FUNCPTR(EFI_STATUS, EFI_GCD_ALLOCATE_TYPE, EFI_GCD_MEMORY_TYPE, UINTN, UINT64, PTR(EFI_PHYSICAL_ADDRESS), EFI_HANDLE, EFI_HANDLE)
EFI_FREE_MEMORY_SPACE				= FUNCPTR(EFI_STATUS, EFI_PHYSICAL_ADDRESS, UINT64)
EFI_REMOVE_MEMORY_SPACE 			= FUNCPTR(EFI_STATUS, EFI_PHYSICAL_ADDRESS, UINT64)
EFI_GET_MEMORY_SPACE_DESCRIPTOR		= FUNCPTR(EFI_STATUS, EFI_PHYSICAL_ADDRESS, PTR(EFI_GCD_MEMORY_SPACE_DESCRIPTOR))
EFI_SET_MEMORY_SPACE_ATTRIBUTES		= FUNCPTR(EFI_STATUS, EFI_PHYSICAL_ADDRESS, UINT64, UINT64)
EFI_GET_MEMORY_SPACE_MAP			= FUNCPTR(EFI_STATUS, PTR(UINTN), PTR(PTR(EFI_GCD_MEMORY_SPACE_DESCRIPTOR)))
EFI_ADD_IO_SPACE					= FUNCPTR(EFI_STATUS, EFI_GCD_IO_TYPE, EFI_PHYSICAL_ADDRESS, UINT64)
EFI_ALLOCATE_IO_SPACE				= FUNCPTR(EFI_STATUS, EFI_GCD_ALLOCATE_TYPE, EFI_GCD_IO_TYPE, UINTN, UINT64, PTR(EFI_PHYSICAL_ADDRESS), EFI_HANDLE, EFI_HANDLE)
EFI_FREE_IO_SPACE					= FUNCPTR(EFI_STATUS, EFI_PHYSICAL_ADDRESS, UINT64)
EFI_REMOVE_IO_SPACE					= FUNCPTR(EFI_STATUS, EFI_PHYSICAL_ADDRESS, UINT64)
EFI_GET_IO_SPACE_DESCRIPTOR			= FUNCPTR(EFI_STATUS, EFI_PHYSICAL_ADDRESS, PTR(EFI_GCD_IO_SPACE_DESCRIPTOR))
EFI_GET_IO_SPACE_MAP				= FUNCPTR(EFI_STATUS, PTR(UINTN), PTR(PTR(EFI_GCD_IO_SPACE_DESCRIPTOR)))
EFI_DISPATCH						= FUNCPTR(EFI_STATUS)
EFI_SCHEDULE						= FUNCPTR(EFI_STATUS, EFI_HANDLE, PTR(EFI_GUID))
EFI_TRUST							= FUNCPTR(EFI_STATUS, EFI_HANDLE, PTR(EFI_GUID))
EFI_PROCESS_FIRMWARE_VOLUME			= FUNCPTR(EFI_STATUS, PTR(VOID), UINTN, PTR(EFI_HANDLE))
EFI_SET_MEMORY_SPACE_CAPABILITIES	= FUNCPTR(EFI_STATUS, EFI_PHYSICAL_ADDRESS, UINT64, UINT64)

class EFI_DXE_SERVICES(STRUCT):
	_fields_ = [
		('Hdr',							EFI_TABLE_HEADER),
		('AddMemorySpace',				EFI_ADD_MEMORY_SPACE),
		('AllocateMemorySpace',			EFI_ALLOCATE_MEMORY_SPACE),
		('FreeMemorySpace',				EFI_FREE_MEMORY_SPACE),
		('RemoveMemorySpace',			EFI_REMOVE_MEMORY_SPACE),
		('GetMemorySpaceDescriptor',	EFI_GET_MEMORY_SPACE_DESCRIPTOR),
		('SetMemorySpaceAttributes',	EFI_SET_MEMORY_SPACE_ATTRIBUTES),
		('GetMemorySpaceMap',			EFI_GET_MEMORY_SPACE_MAP),
		('AddIoSpace',					EFI_ADD_IO_SPACE),
		('AllocateIoSpace',				EFI_ALLOCATE_IO_SPACE),
		('FreeIoSpace',					EFI_FREE_IO_SPACE),
		('RemoveIoSpace',				EFI_REMOVE_IO_SPACE),
		('GetIoSpaceDescriptor',		EFI_GET_IO_SPACE_DESCRIPTOR),
		('GetIoSpaceMap',				EFI_GET_IO_SPACE_MAP),
		('Dispatch',					EFI_DISPATCH),
		('Schedule',					EFI_SCHEDULE),
		('Trust',						EFI_TRUST),
		('ProcessFirmwareVolume',		EFI_PROCESS_FIRMWARE_VOLUME),
		('SetMemorySpaceCapabilities',	EFI_SET_MEMORY_SPACE_CAPABILITIES)
	]

@dxeapi(params={
	"a0": ULONGLONG,
	"a1": ULONGLONG,
	"a2": ULONGLONG,
	"a3": ULONGLONG,
})
def hook_AddMemorySpace(ctx, address, params):
	return EFI_OUT_OF_RESOURCES

@dxeapi(params={
	"a0": ULONGLONG,
	"a1": ULONGLONG,
	"a2": ULONGLONG,
	"a3": ULONGLONG,
	"a4": POINTER, #POINTER_T(ctypes.c_uint64)
	"a5": POINTER, #POINTER_T(None)
	"a6": POINTER, #POINTER_T(None)
})
def hook_AllocateMemorySpace(ctx, address, params):
	return EFI_OUT_OF_RESOURCES

@dxeapi(params={
	"a0": ULONGLONG,
	"a1": ULONGLONG,
})
def hook_FreeMemorySpace(ctx, address, params):
	return EFI_SUCCESS

@dxeapi(params={
	"a0": ULONGLONG,
	"a1": ULONGLONG,
})
def hook_RemoveMemorySpace(ctx, address, params):
	return EFI_SUCCESS

@dxeapi(params={
	"a0": ULONGLONG,
	"a1": POINTER, #POINTER_T(struct_EFI_GCD_MEMORY_SPACE_DESCRIPTOR)
})
def hook_GetMemorySpaceDescriptor(ctx, address, params):
	return EFI_UNSUPPORTED

@dxeapi(params={
	"a0": ULONGLONG,
	"a1": ULONGLONG,
	"a2": ULONGLONG,
})
def hook_SetMemorySpaceAttributes(ctx, address, params):
	return EFI_UNSUPPORTED

@dxeapi(params={
	"a0": POINTER, #POINTER_T(ctypes.c_uint64)
	"a1": POINTER, #POINTER_T(POINTER_T(struct_EFI_GCD_MEMORY_SPACE_DESCRIPTOR))
})
def hook_GetMemorySpaceMap(ctx, address, params):
	return EFI_UNSUPPORTED

@dxeapi(params={
	"a0": ULONGLONG,
	"a1": ULONGLONG,
	"a2": ULONGLONG,
})
def hook_AddIoSpace(ctx, address, params):
	return EFI_OUT_OF_RESOURCES

@dxeapi(params={
	"a0": ULONGLONG,
	"a1": ULONGLONG,
	"a2": ULONGLONG,
	"a3": ULONGLONG,
	"a4": POINTER, #POINTER_T(ctypes.c_uint64)
	"a5": POINTER, #POINTER_T(None)
	"a6": POINTER, #POINTER_T(None)
})
def hook_AllocateIoSpace(ctx, address, params):
	return EFI_OUT_OF_RESOURCES

@dxeapi(params={
	"a0": ULONGLONG,
	"a1": ULONGLONG,
})
def hook_FreeIoSpace(ctx, address, params):
	return EFI_SUCCESS

@dxeapi(params={
	"a0": ULONGLONG,
	"a1": ULONGLONG,
})
def hook_RemoveIoSpace(ctx, address, params):
	return EFI_SUCCESS

@dxeapi(params={
	"a0": ULONGLONG,
	"a1": POINTER, #POINTER_T(struct_EFI_GCD_IO_SPACE_DESCRIPTOR)
})
def hook_GetIoSpaceDescriptor(ctx, address, params):
	return EFI_NOT_FOUND

@dxeapi(params={
	"a0": POINTER, #POINTER_T(ctypes.c_uint64)
	"a1": POINTER, #POINTER_T(POINTER_T(struct_EFI_GCD_IO_SPACE_DESCRIPTOR))
})
def hook_GetIoSpaceMap(ctx, address, params):
	return EFI_OUT_OF_RESOURCES

@dxeapi(params={
})
def hook_Dispatch(ctx, address, params):
	return EFI_SUCCESS

@dxeapi(params={
	"a0": POINTER, #POINTER_T(None)
	"a1": GUID,
})
def hook_Schedule(ctx, address, params):
	return EFI_SUCCESS

@dxeapi(params={
	"a0": POINTER, #POINTER_T(None)
	"a1": GUID,
})
def hook_Trust(ctx, address, params):
	return EFI_NOT_FOUND

@dxeapi(params={
	"a0": POINTER, #POINTER_T(None)
	"a1": ULONGLONG,
	"a2": POINTER, #POINTER_T(POINTER_T(None))
})
def hook_ProcessFirmwareVolume(ctx, address, params):
	return EFI_OUT_OF_RESOURCES

@dxeapi(params={
	"a0": ULONGLONG,
	"a1": ULONGLONG,
	"a2": ULONGLONG,
})
def hook_SetMemorySpaceCapabilities(ctx, address, params):
	return EFI_UNSUPPORTED

def initialize(ql, gDS):
	descriptor = {
		'struct' : EFI_DXE_SERVICES,
		'fields' : (
			('Hdr',							None),
			('AddMemorySpace',				hook_AddMemorySpace),
			('AllocateMemorySpace',			hook_AllocateMemorySpace),
			('FreeMemorySpace',				hook_FreeMemorySpace),
			('RemoveMemorySpace',			hook_RemoveMemorySpace),
			('GetMemorySpaceDescriptor',	hook_GetMemorySpaceDescriptor),
			('SetMemorySpaceAttributes',	hook_SetMemorySpaceAttributes),
			('GetMemorySpaceMap',			hook_GetMemorySpaceMap),
			('AddIoSpace',					hook_AddIoSpace),
			('AllocateIoSpace',				hook_AllocateIoSpace),
			('FreeIoSpace',					hook_FreeIoSpace),
			('RemoveIoSpace',				hook_RemoveIoSpace),
			('GetIoSpaceDescriptor',		hook_GetIoSpaceDescriptor),
			('GetIoSpaceMap',				hook_GetIoSpaceMap),
			('Dispatch',					hook_Dispatch),
			('Schedule',					hook_Schedule),
			('Trust',						hook_Trust),
			('ProcessFirmwareVolume',		hook_ProcessFirmwareVolume),
			('SetMemorySpaceCapabilities',	hook_SetMemorySpaceCapabilities)
		)
	}

	instance = init_struct(ql, gDS, descriptor)
	instance.saveTo(ql, gDS)

__all__ = [
	'EFI_DXE_SERVICES',
	'initialize'
]



#!/usr/bin/env python3
# 
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#

import struct
from qiling.os.windows.const import *
from qiling.os.windows.fncc import *
from qiling.os.windows.utils import *
from qiling.os.windows.thread import *
from qiling.os.windows.handle import *
from qiling.exception import *


# This dll is almost functionally identical to msvcrt, just use hooks from there
from qiling.os.windows.dlls.msvcrt import *



#!/usr/bin/env python3
#
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#


from qiling.const import *
from qiling.os.linux.thread import *
from qiling.const import *
from qiling.os.posix.filestruct import *
from qiling.os.filestruct import *
from qiling.os.posix.const_mapping import *
from qiling.exception import *

def ql_syscall_gettid(ql, *args, **kw):
    if ql.os.thread_management:
      th = ql.os.thread_management.cur_thread
      regreturn = th.id
    else:
      # thread_management is None only if it is a single-threaded process.
      # In single-threaded process, the thread ID is equal to the process ID
      # per Posix documentation.
      regreturn = ql.os.pid
    return regreturn



#!/usr/bin/env python3
# 
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#

from ..ProcessorBind import *
from ..UefiBaseType import *
from ..UefiSpec import EFI_SYSTEM_TABLE, EFI_DEVICE_PATH_PROTOCOL, EFI_IMAGE_UNLOAD
from ..UefiMultiPhase import EFI_MEMORY_TYPE

class EFI_LOADED_IMAGE_PROTOCOL(STRUCT):
	_fields_ = [
		('Revision',		UINT32),
		('PADDING_0',		UINT8 * 4),
		('ParentHandle',	EFI_HANDLE),
		('SystemTable',		PTR(EFI_SYSTEM_TABLE)),
		('DeviceHandle',	EFI_HANDLE),
		('FilePath',		PTR(EFI_DEVICE_PATH_PROTOCOL)),
		('Reserved', 		PTR(VOID)),
		('LoadOptionsSize',	UINT32),
		('PADDING_1',		UINT8 * 4),
		('LoadOptions',		PTR(VOID)),
		('ImageBase',		PTR(VOID)),
		('ImageSize',		UINT64),
		('ImageCodeType',	EFI_MEMORY_TYPE),
		('ImageDataType',	EFI_MEMORY_TYPE),
		('Unload',			EFI_IMAGE_UNLOAD)
	]

def make_descriptor(fields):
	descriptor = {
		"guid" : "5b1b31a1-9562-11d2-8e3f-00a0c969723b",
		"struct" : EFI_LOADED_IMAGE_PROTOCOL,
		"fields" : (
			('Revision',		0x1000),
			('ParentHandle',	0),
			('SystemTable',		fields['gST']),
			('DeviceHandle',	fields['image_base']),
			('FilePath',		0),		# This is a handle to a complex path object, skip it for now.
			('LoadOptionsSize',	0),
			('LoadOptions',		0),
			('ImageBase',		fields['image_base']),
			('ImageSize',		fields['image_size']),
			('ImageCodeType',	EFI_MEMORY_TYPE.EfiLoaderCode),
			('ImageDataType',	EFI_MEMORY_TYPE.EfiLoaderData),
			('Unload',			0)
		)
	}

	return descriptor

__all__ = [
	'EFI_LOADED_IMAGE_PROTOCOL',
	'make_descriptor'
]


#!/usr/bin/env python3
# 
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#

from qiling.os.linux.fncc import *


@linux_kernel_api(params={
    "format": STRING,
})
def hook_printk(ql, address, params):
    return 0


@linux_kernel_api(params={
    "Ptr": POINTER
})
def hook___fentry__(ql, address, params):
    return 0


@linux_kernel_api(params={
    "Ptr": POINTER
})
def hook_mcount(ql, address, params):
    return 0


@linux_kernel_api(params={
    "Ptr": POINTER
})
def hook___x86_indirect_thunk_rax(ql, address, params):
    return 0


@linux_kernel_api(params={
    "Ptr": POINTER
})
def hook__copy_to_user(ql, address, params):
    return 0


@linux_kernel_api(params={
    "Ptr": POINTER
})
def hook__copy_from_user(ql, address, params):
    return 0


@linux_kernel_api(params={
    "Ptr": POINTER
})
def hook___x86_indirect_thunk_r14(ql, address, params):
    return 0


@linux_kernel_api(params={
    "Ptr": POINTER
})
def hook___stack_chk_fail(ql, address, params):
    return 0


@linux_kernel_api(params={
    "Ptr": POINTER
})
def hook_mutex_lock_interruptible(ql, address, params):
    return 0


@linux_kernel_api(params={
    "Ptr": POINTER
})
def hook_mutex_unlock(ql, address, params):
    return 0


@linux_kernel_api(params={
    "Ptr": POINTER
})
def hook_kmalloc_caches(ql, address, params):
    pass


@linux_kernel_api(params={
    "size": SIZE_T,
    "flags": INT
})
def hook_kmalloc(ql, address, params):
    size = params['size']
    addr = ql.heap.alloc(size)
    return addr


@linux_kernel_api(params={
    "dest": POINTER,
    "c": INT,
    "count": SIZE_T
})
def hook_memset(ql, address, params):
    dest = params["dest"]
    c = params["c"]
    count = params["count"]
    ql.mem.write(dest, bytes(c) * count)
    return dest


@linux_kernel_api(params={
    "Ptr": POINTER
})
def hook_kmem_cache_alloc_trace(ql, address, params):
    pass


@linux_kernel_api(params={
    "Ptr": POINTER
})
def hook_get_by_key(ql, address, params):
    pass


@linux_kernel_api(params={
    "Ptr": POINTER
})
def hook_kfree(ql, address, params):
    pass


@linux_kernel_api(params={
    "Ptr": POINTER
})
def hook_misc_register(ql, address, params):
    return 0


@linux_kernel_api(params={
    "Ptr": POINTER
})
def hook_misc_deregister(ql, address, params):
    return 0


@linux_kernel_api(params={
    "Ptr": POINTER
})
def hook___this_module(ql, address, params):
    pass


@linux_kernel_api(params={
    "Ptr": POINTER
})
def hook_htable(ql, address, params):
    pass


@linux_kernel_api(params={
    "Ptr": POINTER
})
def hook_strcpy(ql, address, params):
    pass


@linux_kernel_api(params={
    "major": INT,
    "baseminor": INT,
    "count": INT,
    "name": STRING,
    "fops": POINTER,
})
def hook___register_chrdev(ql, address, params):
    return 0


@linux_kernel_api(params={
    "major": INT,
    "name": STRING,
    "fops": POINTER,
})
def hook_register_chrdev(ql, address, params):
    return 0


@linux_kernel_api(params={
    "Ptr": POINTER
})
def hook___class_create(ql, address, params):
    return 0


@linux_kernel_api(params={
    "Ptr": POINTER,
    "name": STRING,
})
def hook___unregister_chrdev(ql, address, params):
    return 0


@linux_kernel_api(params={
    "Ptr": POINTER
})
def hook_device_create(ql, address, params):
    return 0


@linux_kernel_api(params={
    "Ptr": POINTER
})
def hook_class_destroy(ql, address, params):
    return 0


@linux_kernel_api(params={
    "Ptr": POINTER
})
def hook_device_destroy(ql, address, params):
    return 0


@linux_kernel_api(params={
    "Ptr": POINTER
})
def hook_class_unregister(ql, address, params):
    return 0


@linux_kernel_api(params={
    "dev": POINTER
})
def hook_dev_get_flags(ql, address, params):
    return 0


@linux_kernel_api(params={
    "buffer": POINTER,
    "format": STRING
})
def hook_sprintf(ql, address, params):
    return 0


@linux_kernel_api(params={
    "str1": STRING,
    "str2": STRING
})
def hook_strcmp(ql, address, params):
    return 0


@linux_kernel_api(params={
    "str1": STRING,
    "str2": STRING,
    "count": SIZE_T
})
def hook_strncmp(ql, address, params):
    s1 = params["str"]
    s2 = params["str2"]
    count = params["count"]
    string1 = s1[:count]
    string2 = s2[:count]

    if string1 == string2:
        result = 0
    elif string1 > string2:
        result = 1
    else:
        result = -1
    return result


@linux_kernel_api(params={
    "haystack": STRING,
    "needle": STRING
})
def hook_strstr(ql, address, params):
    _haystack = params["haystack"]
    _needle = params["needle"]
    index = _haystack.find(_needle)
    return 0 if index == -1 else index


@linux_kernel_api(params={
    "str": STRING,
})
def hook_strlen(ql, address, params):
    _str = params["str"]
    strlen = len(_str)
    return strlen


@linux_kernel_api(params={
    "s1": POINTER,
    "s2": POINTER,
    "size": SIZE_T
})
def hook_memcmp(ql, address, params):
    s1 = params['s1']
    s2 = params['s2']
    size = params['size']
    return ql.mem.read(s1, size) ==  ql.mem.read(s2, size)


@linux_kernel_api(params={
    "str1": POINTER,
    "str2": POINTER,
    "base": INT
})
def hook_simple_strtol(ql, address, params):
    return 0


@linux_kernel_api(params={
})
def hook_pv_cpu_ops(ql, address, params):
    return 0


@linux_kernel_api(params={
})
def hook_sock_create(ql, address, params):
    return 0


@linux_kernel_api(params={
})
def hook_sock_release(ql, address, params):
    return 0


@linux_kernel_api(params={
})
def hook_filp_open(ql, address, params):
    return 0


@linux_kernel_api(params={
})
def hook_filp_close(ql, address, params):
    return 0


@linux_kernel_api(params={
})
def hook_PDE_DATA(ql, address, params):
    return 0


@linux_kernel_api(params={
})
def hook_hook__kmalloc(ql, address, params):
    return 0


@linux_kernel_api(params={
})
def hook_hook_raw_spin_lock_irqsave(ql, address, params):
    return 0


@linux_kernel_api(params={
})
def hook_hook_raw_spin_unlock_irqrestore(ql, address, params):
    return 0


@linux_kernel_api(params={
})
def hook_hook_current_task(ql, address, params):
    return 0


@linux_kernel_api(params={
})
def hook_hook_prepare_kernel_cred(ql, address, params):
    return 0


@linux_kernel_api(params={
})
def hook_hook_commit_creds(ql, address, params):
    return 0


@linux_kernel_api(params={
})
def hook_hook_kallsyms_on_each_symbol(ql, address, params):
    return 0


@linux_kernel_api(params={
})
def hook_hook_register_module_notifier(ql, address, params):
    return 0


@linux_kernel_api(params={
})
def hook_hook_unregister_module_notifier(ql, address, params):
    return 0


@linux_kernel_api(params={
})
def hook_hook_kobject_del(ql, address, params):
    return 0

@linux_kernel_api(params={
})
def hook_proc_create(ql, address, params):
    # indicate that proc_create successed, and return (proc_dir_entry *)
    return 0x1000

@linux_kernel_api(params={
})
def hook_remove_proc_entry(ql, address, params):
    return 0

@linux_kernel_api(params={
    "fd": UINT,
    "buf": STRING,
    "count": SIZE_T,
})
def hook_sys_read(ql, address, params):
    return 0

@linux_kernel_api(params={
    "fd": UINT,
    "buf": STRING,
    "count": SIZE_T,
})
def hook_sys_write(ql, address, params):
    return 0

@linux_kernel_api(params={
    "pathname": STRING,
    "flags": UINT,
})
def hook_sys_open(ql, address, params):
    return 0

@linux_kernel_api(params={
})
def hook_make_kgid(ql, address, params):
    return 0

@linux_kernel_api(params={
})
def hook_make_kuid(ql, address, params):
    return 0

@linux_kernel_api(params={
})
def hook_prepare_creds(ql, address, params):
    # return non-NULL creds
    return 0x1000

@linux_kernel_api(params={
})
def hook_commit_creds(ql, address, params):
    return 0

@linux_kernel_api(params={
})
def hook_abort_creds(ql, address, params):
    return 0

@linux_kernel_api(params={
    "dest": POINTER,
    "src": POINTER,
    "size": SIZE_T
})
def hook_memcpy(ql, address, params):
    dest = params["dest"]
    src = params["src"]
    size = params["size"]
    ql.mem.write(dest, bytes(ql.mem.read(src, size)))
    return dest



#!/usr/bin/env python3
# 
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#

from elftools.elf.elffile import ELFFile

from qiling import Qiling
from qiling.const import *
from capstone import *


class QlDisassember():
    def __init__(self, ql:Qiling):
        self.ql = ql

    def disasm_all_lines(self):
        disasm_result = []

        if self.ql.ostype == QL_OS.LINUX:
            disasm_result = self.disasm_elf()

        return disasm_result

    def disasm_elf(self, seg_name='.text'):
        def disasm(ql, address, size):
            md = ql.create_disassembler()
            md.detail = True
            return md.disasm(ql.mem.read(address, size), address)

        disasm_result = []
        if self.ql._archtype == QL_ARCH.X86:
            BASE = int(self.ql.profile.get("OS32", "load_address"), 16)
            seg_start = 0x0
            seg_end = 0x0

            f = open(self.ql.path, 'rb')
            elffile = ELFFile(f)
            elf_header = elffile.header
            reladyn = elffile.get_section_by_name(seg_name)

            # No PIE
            if elf_header['e_type'] == 'ET_EXEC':
                seg_start = reladyn.header.sh_addr
                seg_end = seg_start + reladyn.data_size
            # PIE
            elif elf_header['e_type'] == 'ET_DYN':
                seg_start = BASE + reladyn.header.sh_addr
                seg_end = seg_start + reladyn.data_size

            for insn in disasm(ql, seg_start, seg_end-seg_start):
                disasm_result.append(insn)       

        return disasm_result



#!/usr/bin/env python3
# 
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#

import ctypes, socket, struct
 
 
from functools import wraps
from unicorn.x86_const import *
from unicorn import UcError, UC_ERR_READ_UNMAPPED, UC_ERR_FETCH_UNMAPPED

from qiling.os.macos.structs import *
from qiling.os.macos.utils import gen_stub_code
from .macos_structs import *


class QlMacOSEv:
    def __init__(self, ql, ev_type, ev_name, ev_obj, ev_obj_idx=-1, protocol=None):
        self.ql = ql
        self.type = ev_type
        self.name = ev_name
        self.event = ev_obj
        self.params = []
        self.ev_index = ev_obj_idx
        self.protocol = protocol

    def set_params(self, params):
        if self.ev_index != -1 and self.event is not None:
            self.params = params[:]
            self.params.insert(self.ev_index, self.event.base)
        else:
            self.params = params[:]

    def dump(self):
        self.ql.log.info("[*] Dumping object: %s with type %d" % (self.name, self.type.value))
        for field in self.event._fields_:
            if isinstance(getattr(self.event, field[0]), POINTER64):
                self.ql.log.info("%s: 0x%x" % (field[0], getattr(self.event, field[0]).value))
            elif isinstance(getattr(self.event, field[0]), int):
                self.ql.log.info("%s: %d" % (field[0], getattr(self.event, field[0])))
            elif isinstance(getattr(self.event, field[0]), bytes):
                self.ql.log.info("%s: %s" % (field[0], getattr(self.event, field[0]).decode()))

def init_ev_ctx(f):
    @wraps(f)
    def wrapper(self, *args, **kw):
        if self.label is None:
            label_addr = self.ql.os.heap.alloc(ctypes.sizeof(label_t))
            self.label = label_t(self.ql, label_addr)
            self.label.l_flags = 1
            self.label.updateToMem()

        if self.cred is None:
            cred_addr = self.ql.os.heap.alloc(ctypes.sizeof(ucred_t))
            self.cred = ucred_t(self.ql, cred_addr)
            self.cred.cr_ref = 2
            pyarr = [20, 12, 61, 79, 80, 81, 98, 33, 100, 204, 250, 395, 398, 399, 701, 0]
            self.cred.cr_posix = ucred_t.posix_cred_t(
                501,
                501,
                501,
                15,
                (ctypes.c_uint32 * len(pyarr))(*pyarr),
                20,
                20,
                501,
                2)
            self.cred.cr_label = POINTER64(self.label.base)
            self.cred.updateToMem()

        if self.vnode is None:
            tmp_addr = self.ql.os.heap.alloc(ctypes.sizeof(vnode_t))
            self.vnode = vnode_t(self.ql, tmp_addr)
            tmp_name = self.ql.os.heap.alloc(len(self.current_proc))
            self.ql.mem.write(tmp_name, self.current_proc.encode())
            self.vnode.v_name = POINTER64(tmp_name)
            self.vnode.updateToMem()

        return f(self, *args, **kw)
    return wrapper

class QlMacOSEvManager:
    def __init__(self, ql):
        self.ql = ql
        self.callbacks = {}
        self.jobs = []

        self.src_host = "192.168.13.37"
        self.src_port = socket.htons(1337)
        self.src_mac = b"\xba\xbe\xc0\xde\xbe\x57"
        
        self.dst_host = "10.2.13.38"
        self.dst_port = socket.htons(1338)
        self.dst_mac = b"\xba\xbe\xfe\xed\xfa\xce"

        self.current_proc = self.ql.argv[0]
        self.cred = None
        self.label = None
        self.vnode = None

        self.target_pid = 0xdeadbeef

        self.my_procs = []
        self.allproc = None

        self.map_fd = {}
        self.ipf_cookie = {}
        self.sockets = []

        self.deadcode = None

    def add_process(self, pid, name):
        for p in self.my_procs:
            if p.p_pid == pid:
                self.ql.log.info("Duplicated process")
                return

        cur_proc_addr = self.ql.os.heap.alloc(ctypes.sizeof(proc_t))
        cur_proc = proc_t(self.ql, cur_proc_addr)
        cur_proc.p_pid = pid
        cur_proc.p_ppid = 1
        cur_proc.p_pgrpid = pid
        cur_proc.p_flag = 0x00000004 # 64 bit proccess
        cur_proc.p_uid = 0
        cur_proc.p_gid = 0
        cur_proc.p_ruid = 0
        cur_proc.p_rgid = 0
        cur_proc.p_svuid = 0
        cur_proc.p_svgid = 0
        p_comm = name.encode() + b"\x00" * (17 - len(name))
        cur_proc.p_comm = p_comm
        p_name = p_comm + b"\x00" * 16
        cur_proc.p_name = p_name

        cred_addr = self.ql.os.heap.alloc(ctypes.sizeof(ucred_t))
        cur_proc.p_ucred = POINTER64(cred_addr)

        cur_proc.updateToMem()

        if len(self.my_procs) == 0:
            self.allproc.lh_first = POINTER64(cur_proc_addr)
            self.allproc.updateToMem()
            self.my_procs.append(cur_proc)
        else:
            prev_proc = self.my_procs[-1]
            prev_proc.p_list.le_next = POINTER64(cur_proc_addr)
            cur_proc.p_list.le_prev = POINTER64(prev_proc.base)
            self.my_procs.append(cur_proc)
            cur_proc.updateToMem()
            prev_proc.updateToMem()

    def proc_find(self, pid):
        for p in self.my_procs:
            if p.p_pid == pid:
                return p
        return None

    def set_allproc(self, allprocs_addr):
        self.my_procs.clear()
        self.allproc = list_head(self.ql, allprocs_addr).loadFromMem()

    def set_target_pid(self, target_pid):
        self.target_pid = target_pid

    def set_proc(self, name):
        self.current_proc = name

    def set_src_host(self, host):
        self.src_host = host

    def set_src_port(self, port):
        self.src_port = htons(port)

    def set_src_mac(self, mac):
        self.src_mac = mac

    def set_dst_host(self, host):
        self.dst_host = host

    def set_dst_port(self, port):
        self.dst_port = htons(port)

    def set_dst_mac(self, mac):
        self.dst_mac = mac

    def register(self, func_addr, ev_name, ev_type, ev_obj=None, idx=-1, protocol=None):
        ev_name = ev_name.decode()
        event = QlMacOSEv(self.ql, ev_type, ev_name, ev_obj, idx, protocol)
        if event in self.callbacks:
            self.callbacks[event].append(func_addr)
        else:
            self.callbacks[event] = [func_addr]

    def deregister(self, ev_name, keyword=b""):
        found = self.get_events_by_name(ev_name, keyword)
        for f in found:
            del self.callbacks[f]

    def get_events_by_name(self, ev_name, keyword=b""):
        found = []
        for ev, cb in self.callbacks.items():
            if (len(ev_name) > 0 and ev.name == ev_name) or (len(keyword) > 0 and keyword in ev.name):
                found.append(ev)
        return found

    def emit(self, ev_name, ev_type, params):
        found = self.get_event_by_name_and_type(ev_name, ev_type)
        if found is None:
            self.ql.log.info("No callbacks found for (%s, %s)" % (ev_name, ev_type))
            return

        found.set_params(params)
        for cb in self.callbacks[found]:
            if self.ql.os.RUN is True:
                self.ql.stack_push(gen_stub_code(self.ql, found.params, cb))
            else:
                self.jobs.append((cb, found))

    def emit_by_type(self, ev_type, params, ins_cookie=False):
        found = self.get_events_by_type(ev_type)
        for ev in found:
            if ins_cookie is True and ev.name in self.ipf_cookie:
                params[0] = self.ipf_cookie[ev.name]
            ev.set_params(params)
            for cb in self.callbacks[ev]:
                if self.ql.os.RUN is True:
                    self.ql.stack_push(gen_stub_code(self.ql, ev.params, cb))
                else:
                    self.jobs.append((cb, ev))

    def emit_by_type_and_proto(self, ev_type, protocol, params):
        found = self.get_events_by_type_and_proto(ev_type, protocol)
        for ev in found:
            ev.set_params(params)
            for cb in self.callbacks[ev]:
                if self.ql.os.RUN is True:
                    self.ql.stack_push(gen_stub_code(self.ql, ev.params, cb))
                else:
                    self.jobs.append((cb, ev))

    def get_events_by_type_and_proto(self, ev_type, protocol):
        found = []
        for ev, cb in self.callbacks.items():
            if ev.type == ev_type and ev.protocol == protocol:
                found.append(ev)
        return found

    def get_events_by_type(self, ev_type):
        found = []
        for ev, cb in self.callbacks.items():
            if ev.type == ev_type:
                found.append(ev)
        return found

    def get_event_by_name_and_type(self, ev_name, ev_type):
        found = None
        for ev, cb in self.callbacks.items():
            if ev.name == ev_name and ev.type == ev_type:
                found = ev
                break
        return found

    def trigger(self):
        reg_list = [UC_X86_REG_RDI, UC_X86_REG_RSI, UC_X86_REG_RDX, UC_X86_REG_RCX, UC_X86_REG_R8, UC_X86_REG_R9]
        for cb, ev in self.jobs:
            params = ev.params
            remains = 0
            if len(params) <= 6:
                for idx, p in enumerate(params):
                    self.ql.uc.reg_write(reg_list[idx], p)
            else:
                for idx, p in enumerate(params[:6]):
                    self.ql.uc.reg_write(reg_list[idx], p)
                remains = len(params) - 6
                for i in range(remains):
                    self.ql.stack_push(params[6 + i])
            # TODO: Too many kind of callbacks to find saved rip, lmao
            self.ql.os.savedrip=self.deadcode
            self.ql.run(begin=cb)

        self.jobs.clear()

    def clear_heap(self):
        self.ql.os.heap.clear()

    @init_ev_ctx
    def sysctlbyname(self, name, namelen, oldp, oldlenp, new, newlen):
        uap_addr = self.ql.os.heap.alloc(ctypes.sizeof(sysctlbyname_args_t))
        uap = sysctlbyname_args_t(self.ql, uap_addr)

        uap.name = POINTER64(name)
        uap.namelen = namelen
        uap.oldp = POINTER64(oldp)
        uap.oldlenp = POINTER64(oldlenp)
        uap.new = POINTER64(new)
        uap.newlen = newlen
        uap.updateToMem()

        self.ql.reg.rdi = self.proc_find(0x1337).base
        self.ql.reg.rsi = uap_addr # uap
        self.ql.reg.rdx = 0 # unused retval
        self.ql.os.savedrip=self.deadcode
        self.ql.run(self.ql.loader.kernel_extrn_symbols_detail[b"_sysctlbyname"]["n_value"])

    @init_ev_ctx
    def ctl_connect(self, nke_name, unitinfo=0):
        nke_addr = self.ql.os.heap.alloc(ctypes.sizeof(sockaddr_ctl_t))
        nke_obj = sockaddr_ctl_t(self.ql, nke_addr)

        nke_obj.sc_len = ctypes.sizeof(sockaddr_ctl_t)
        nke_obj.sc_family = 32 # AF_SYSTEM
        nke_obj.ss_sysaddr = 2 # AF_SYS_CONTROL
        nke_obj.sc_id = 0x1337
        nke_obj.sc_unit = 2
        nke_obj.updateToMem()

        # static int ctl_connect(kern_ctl_ref ctl_ref, struct sockaddr_ctl *sac, void **unitinfo)
        self.emit(nke_name, MacOSEventType.EV_CTL_CONNECT, [nke_addr, unitinfo])

    @init_ev_ctx
    def ctl_disconnect(self, nke_name, unitinfo=0):

        # static int ctl_disconnect(kern_ctl_ref ctl_ref, u_int32_t unit, void *unitinfo)
        self.emit(nke_name, MacOSEventType.EV_CTL_DISCONNECT, [2, unitinfo])

    @init_ev_ctx
    def ctl_send(self, nke_name, data):
        buf = self.ql.os.heap.alloc(len(data))
        self.ql.mem.write(buf, data.encode())
        mbuf_addr = self.ql.os.heap.alloc(ctypes.sizeof(mbuf_t))
        mbuf = mbuf_t(self.ql, mbuf_addr)
        mbuf.m_hdr.mh_len = len(data)
        mbuf.m_hdr.mh_data = POINTER64(buf)
        mbuf.m_hdr.mh_flags = 2
        mbuf.updateToMem()

        # static int ctl_send(kern_ctl_ref ctlref, unsigned int unit, void *userdata, mbuf_t m, int flags)
        self.emit(nke_name, MacOSEventType.EV_CTL_SEND, [2, 0, mbuf_addr, 2])

    @init_ev_ctx
    def ctl_setopt(self, nke_name, opt, data, unitinfo=0):
        sopt_val = self.ql.os.heap.alloc(len(data))
        self.ql.mem.write(sopt_val, data.encode())

        # static int ctl_setopt(kern_ctl_ref ctl_ref, u_int32_t unit, void *unitinfo, int opt, void *data, size_t len)
        self.emit(nke_name, MacOSEventType.EV_CTL_SETOPT, [2, unitinfo, opt, sopt_val, len(data)])

    @init_ev_ctx
    def ctl_getopt(self, nke_name, opt, data, unitinfo=0):
        sopt_val = self.ql.os.heap.alloc(len(data))
        self.ql.mem.write(sopt_val, data.encode())

        # static int ctl_getopt(kern_ctl_ref ctl_ref, u_int32_t unit, void *unitinfo, int opt, void *data, size_t *len)
        self.emit(nke_name, MacOSEventType.EV_CTL_GETOPT, [2, unitinfo, opt, sopt_val, len(data)])

    @init_ev_ctx
    def net_new_socket(self, protocol, cookie_holder=0):
        if cookie_holder == 0:
            cookie_holder = self.ql.os.heap.alloc(8)
        # sflt_attach(void **cookie, socket_t socket)
        self.emit_by_type_and_proto(MacOSEventType.EV_SFLT_ATTACH, protocol.value, [cookie_holder, 0])
        self.trigger()
        cookie = struct.unpack("<Q", self.ql.mem.read(cookie_holder, 8))[0]
        return cookie

    def gen_sock(self, host, port):
        # Currently support only AF_INET
        sock_addr = self.ql.os.heap.alloc(ctypes.sizeof(sockaddr_in_t))
        sock = sockaddr_in_t(self.ql, sock_addr)

        sock.sin_len = ctypes.sizeof(sockaddr_in_t)
        sock.sin_family = 2 # AF_INET
        sock.sin_port = port
        sock.sin_addr = sockaddr_in_t.in_addr_t(struct.unpack("I", socket.inet_aton(host))[0])
        sock.updateToMem()
        return sock

    def clear_socks(self):
        self.sockets.clear()

    @init_ev_ctx
    def net_connect_out(self, protocol):
        from_sock = self.gen_sock(self.src_host, self.src_port)
        from_idx = len(self.sockets)
        self.sockets.append(from_sock)

        to_sock = self.gen_sock(self.dst_host, self.dst_port)
        to_idx = len(self.sockets)
        self.sockets.append(to_sock)

        # sflt_connect_out(void *cookie, socket_t socket, const struct sockaddr *to)
        self.emit_by_type_and_proto(MacOSEventType.EV_SFLT_CONNECT_OUT, protocol.value, [cookie, from_idx, to_sock.base])

        # sflt_notify(void *cookie, socket_t socket, sflt_event_t event, void *param)
        self.emit_by_type_and_proto(MacOSEventType.EV_SFLT_NOTIFY_BOUND, protocol.value, [cookie, from_idx, SocketEvent.BOUND.value - base_event_socket, 0])

        # sflt_notify(void *cookie, socket_t socket, sflt_event_t event, void *param)
        self.emit_by_type_and_proto(MacOSEventType.EV_SFLT_NOTIFY_CONNECTING, protocol.value, [cookie, from_idx, SocketEvent.CONNECTING.value - base_event_socket, 0])

        # sflt_notify(void *cookie, socket_t socket, sflt_event_t event, void *param)
        self.emit_by_type_and_proto(MacOSEventType.EV_SFLT_NOTIFY_CONNECTED, protocol.value, [cookie, to_idx, SocketEvent.CONNECTED.value - base_event_socket, 0])

        return from_idx, to_idx

    @init_ev_ctx
    def net_connect_in(self, cookie, protocol):
        from_sock = self.gen_sock(self.src_host, self.src_port)
        from_idx = len(self.sockets)
        self.sockets.append(from_sock)

        to_sock = self.gen_sock(self.dst_host, self.dst_port)
        to_idx = len(self.sockets)
        self.sockets.append(to_sock)

        # sflt_bind(void *cookie, socket_t socket, const struct sockaddr *to)
        self.emit_by_type_and_proto(MacOSEventType.EV_SFLT_BIND, protocol.value, [cookie, from_idx, from_sock.base])

        # sflt_notify(void *cookie, socket_t socket, sflt_event_t event, void *param)
        self.emit_by_type_and_proto(MacOSEventType.EV_SFLT_NOTIFY_BOUND, protocol.value, [cookie, from_idx, SocketEvent.BOUND.value - base_event_socket, 0])

        # sflt_listen(void *cookie, socket_t socket)
        self.emit_by_type_and_proto(MacOSEventType.EV_SFLT_LISTEN, protocol.value, [cookie, from_idx])

        # sflt_connect_in(void *cookie, socket_t socket, const struct sockaddr *to)
        self.emit_by_type_and_proto(MacOSEventType.EV_SFLT_CONNECT_IN, protocol.value, [cookie, from_idx, to_sock.base])

        # sflt_notify(void *cookie, socket_t socket, sflt_event_t event, void *param)
        self.emit_by_type_and_proto(MacOSEventType.EV_SFLT_NOTIFY_CONNECTED, protocol.value, [cookie, to_idx, SocketEvent.CONNECTING.value - base_event_socket, 0])

        return from_idx, to_idx

    @init_ev_ctx
    def net_disconnect(self, cookie, so, protocol):
        # sflt_detach_ipv4(void *cookie, socket_t socket)
        self.emit_by_type_and_proto(MacOSEventType.EV_SFLT_DETACH, protocol.value, [cookie, 0])
        # ipf_detach(void* cookie)
        self.emit_by_type(MacOSEventType.EV_IPF_DETACH, [cookie])

        # sflt_notify(void *cookie, socket_t socket, sflt_event_t event, void *param)
        self.emit_by_type_and_proto(MacOSEventType.EV_SFLT_NOTIFY_CLOSING, protocol.value, [cookie, so, SocketEvent.CLOSING.value - base_event_socket, 0])

        # sflt_notify(void *cookie, socket_t socket, sflt_event_t event, void *param)
        self.emit_by_type_and_proto(MacOSEventType.EV_SFLT_NOTIFY_DISCONNECTING, protocol.value, [cookie, so, SocketEvent.DISCONNECTING.value - base_event_socket, 0])

        # sflt_notify(void *cookie, socket_t socket, sflt_event_t event, void *param)
        self.emit_by_type_and_proto(MacOSEventType.EV_SFLT_NOTIFY_DISCONNECTED, protocol.value, [cookie, so, SocketEvent.DISCONNECTED.value - base_event_socket, 0])

    @init_ev_ctx
    def net_set_option(self, cookie, so, data, protocol):
        sopt_val = self.ql.os.heap.alloc(len(data))
        sopt_addr = self.ql.os.heap.alloc(ctypes.sizeof(sockopt_t))
        sopt = sockopt_t(self.ql, sopt_addr)
        sopt.sopt_val = POINTER64(sopt_val)
        sopt.updateToMem()

        # sflt_set_option(void *cookie, socket_t socket, sockopt_t option)
        self.emit_by_type_and_proto(MacOSEventType.EV_SFLT_SETOPTION, protocol.value, [cookie, so, sopt_addr])

    @init_ev_ctx
    def net_get_option(self, cookie, so, data, protocol):
        sopt_val = self.ql.os.heap.alloc(len(data))
        sopt_addr = self.ql.os.heap.alloc(ctypes.sizeof(sockopt_t))
        sopt = sockopt_t(self.ql, sopt_addr)
        sopt.sopt_val = POINTER64(sopt_val)
        sopt.updateToMem()

        # sflt_get_option(void *cookie, socket_t socket, sockopt_t option)
        self.emit_by_type_and_proto(MacOSEventType.EV_SFLT_GETOPTION, protocol.value, [cookie, so, sopt_addr])

    @init_ev_ctx
    def net_send(self, cookie, from_idx, to_idx, protocol, last_header, data):
        from scapy.all import Ether, IP, TCP, Raw, ICMP

        ether = Ether(src=self.src_mac, dst=self.dst_mac)
        ip = IP(src=self.src_host, dst=self.dst_host)
        packet_header = (ether).build()
        packet = (ether/ip/last_header/Raw(load=data)).build()

        packet_addr = self.ql.os.heap.alloc(len(packet))
        self.ql.mem.write(packet_addr, packet)

        mbuf_addr = self.ql.os.heap.alloc(ctypes.sizeof(mbuf_t))
        mbuf = mbuf_t(self.ql, mbuf_addr)
        mbuf.m_hdr.mh_len = len(packet) - len(packet_header)
        mbuf.m_hdr.mh_data = POINTER64(packet_addr + len(packet_header))
        mbuf.m_hdr.mh_flags = 2
        mbuf.M_dat.MH.MH_pkthdr.pkt_hdr = POINTER64(packet_addr)
        mbuf.m_hdr.mh_types = 1
        mbuf.M_dat.MH.MH_dat.MH_ext.ext_buf = POINTER64(0)
        mbuf.updateToMem()

        mbuf_holder = self.ql.os.heap.alloc(8)
        self.ql.mem.write(mbuf_holder, struct.pack("<Q", mbuf_addr))

        packet_header = (ether/ip/last_header).build()

        mbuf2_addr = self.ql.os.heap.alloc(ctypes.sizeof(mbuf_t))
        mbuf2 = mbuf_t(self.ql, mbuf2_addr)
        mbuf2.m_hdr.mh_len = len(data)
        mbuf2.m_hdr.mh_data = POINTER64(packet_addr + len(packet_header))
        mbuf2.m_hdr.mh_flags = 2
        mbuf2.M_dat.MH.MH_pkthdr.pkt_hdr = POINTER64(packet_addr)
        mbuf2.m_hdr.mh_type = 1
        mbuf2.M_dat.MH.MH_dat.MH_ext.ext_buf = POINTER64(packet_addr + len(packet_header))
        mbuf2.updateToMem()

        mbuf2_holder = self.ql.os.heap.alloc(8)
        self.ql.mem.write(mbuf2_holder, struct.pack("<Q", mbuf2_addr))

        # sflt_data_out(void *cookie, socket_t socket, const struct sockaddr *to, mbuf_t *data, mbuf_t *control, sflt_data_flag_t flags)
        self.emit_by_type_and_proto(MacOSEventType.EV_SFLT_DATA_OUT, protocol.value, [cookie, from_idx, self.sockets[to_idx].base, mbuf2_holder, 0, 0])
        # ipf_output(void* cookie, mbuf_t *data, ipf_pktopts_t options)
        self.emit_by_type(MacOSEventType.EV_IPF_OUTPUT, [0, mbuf_holder, 0], ins_cookie=True)

    @init_ev_ctx
    def net_recv(self, cookie, from_idx, to_idx, protocol, last_header, data):
        from scapy.all import Ether, IP, TCP, Raw, ICMP

        ether = Ether(src=self.src_mac, dst=self.dst_mac)
        ip = IP(src=self.src_host, dst=self.dst_host)
        packet_header = (ether).build()
        packet = (ether/ip/last_header/Raw(load=data)).build()

        packet_addr = self.ql.os.heap.alloc(len(packet))
        self.ql.mem.write(packet_addr, packet)

        mbuf_addr = self.ql.os.heap.alloc(ctypes.sizeof(mbuf_t))
        mbuf = mbuf_t(self.ql, mbuf_addr)
        mbuf.m_hdr.mh_len = len(packet) - len(packet_header)
        mbuf.m_hdr.mh_data = POINTER64(packet_addr + len(packet_header))
        mbuf.m_hdr.mh_flags = 2
        mbuf.M_dat.MH.MH_pkthdr.pkt_hdr = POINTER64(packet_addr)
        mbuf.m_hdr.mh_type = 1
        mbuf.M_dat.MH.MH_dat.MH_ext.ext_buf = POINTER64(0)
        mbuf.updateToMem()

        mbuf_holder = self.ql.os.heap.alloc(8)
        self.ql.mem.write(mbuf_holder, struct.pack("<Q", mbuf_addr))

        packet_header = (ether/ip/last_header).build()

        mbuf2_addr = self.ql.os.heap.alloc(ctypes.sizeof(mbuf_t))
        mbuf2 = mbuf_t(self.ql, mbuf2_addr)
        mbuf2.m_hdr.mh_len = len(data)
        mbuf2.m_hdr.mh_data = POINTER64(packet_addr + len(packet_header))
        mbuf2.m_hdr.mh_flags = 2
        mbuf2.M_dat.MH.MH_pkthdr.pkt_hdr = POINTER64(packet_addr)
        mbuf2.m_hdr.mh_type = 1
        mbuf2.M_dat.MH.MH_dat.MH_ext.ext_buf = POINTER64(packet_addr + len(packet_header))
        mbuf2.updateToMem()

        mbuf2_holder = self.ql.os.heap.alloc(8)
        self.ql.mem.write(mbuf2_holder, struct.pack("<Q", mbuf2_addr))

        # sflt_data_in(void *cookie, socket_t socket, const struct sockaddr *to, mbuf_t *data, mbuf_t *control, sflt_data_flag_t flags)
        self.emit_by_type_and_proto(MacOSEventType.EV_SFLT_DATA_IN, protocol.value, [cookie, from_idx, self.sockets[to_idx].base, mbuf2_holder, 0, 0])
        # ipf_input(void* cookie, mbuf_t *data, int offset, u_int8_t protocol)
        self.emit_by_type(MacOSEventType.EV_IPF_INPUT, [0, mbuf_holder, 0, protocol.value], ins_cookie=True)

    @init_ev_ctx
    def kauth_generic(self, action):
        self.emit("KAUTH_GENERIC", EV_KAUTH_GENERIC, [self.cred.base, 0, action.value, 0, 0, 0, 0])

    @init_ev_ctx
    def kauth_process(self, action):
        signal_addr = self.ql.os.heap.alloc(ctypes.sizeof(c_int))
        self.emit("KAUTH_PROCESS", EV_KAUTH_PROCESS, [self.cred.base, 0, action.value, 0, signal_addr, 0, 0])
        return struct.unpack("<I", signal_addr)

    @init_ev_ctx
    def kauth_vnode(self, action, parent_dir):
        tmp_addr = self.ql.os.heap.alloc(ctypes.sizeof(vnode_t))
        parent_vnode = vnode_t(self.ql, tmp_addr)
        tmp_name = self.ql.os.heap.alloc(len(parent_dir))
        self.ql.mem.write(tmp_name, parent_dir.encode())
        parent_vnode.v_name = POINTER64(tmp_name)
        parent_vnode.updateToMem()

        self.emit("KAUTH_VNODE", EV_KAUTH_VNODE, [self.cred.base, 0, action.value, 0, self.vnode.base, tmp_addr])

# arguments passed to KAUTH_FILEOP_OPEN listeners
#           arg0 is pointer to vnode (vnode *) for given user path.
# 	    arg1 is pointer to path (char *) passed in to open.
# arguments passed to KAUTH_FILEOP_CLOSE listeners
#           arg0 is pointer to vnode (vnode *) for file to be closed.
# 	    arg1 is pointer to path (char *) of file to be closed.
# 	    arg2 is close flags.
# arguments passed to KAUTH_FILEOP_WILL_RENAME listeners
# 	    arg0 is pointer to vnode (vnode *) of the file being renamed
# 	    arg1 is pointer to the "from" path (char *)
# 	    arg2 is pointer to the "to" path (char *)
# arguments passed to KAUTH_FILEOP_RENAME listeners
# 	    arg0 is pointer to "from" path (char *).
# 	    arg1 is pointer to "to" path (char *).
# arguments passed to KAUTH_FILEOP_EXCHANGE listeners
# 	    arg0 is pointer to file 1 path (char *).
# 	    arg1 is pointer to file 2 path (char *).
# arguments passed to KAUTH_FILEOP_LINK listeners
# 	    arg0 is pointer to path to file we are linking to (char *).
# 	    arg1 is pointer to path to the new link file (char *).
# arguments passed to KAUTH_FILEOP_EXEC listeners
# 	    arg0 is pointer to vnode (vnode *) for executable.
# 	    arg1 is pointer to path (char *) to executable.
# arguments passed to KAUTH_FILEOP_DELETE listeners
# 	    arg0 is pointer to vnode (vnode *) of file/dir that was deleted.
# 	    arg1 is pointer to path (char *) of file/dir that was deleted.
    @init_ev_ctx
    def kauth_fileop(self, action, params={}):
        path = self.ql.os.heap.alloc(len(self.current_proc) + 1)
        self.ql.mem.write(path, self.current_proc.encode() + b"\x00")
        if action == Kauth.KAUTH_FILEOP_OPEN:
            self.emit("KAUTH_FILEOP", MacOSEventType.EV_KAUTH_FILEOP, [self.cred.base, 0, action.value, self.vnode.base, path, 0, 0])
        elif action == Kauth.KAUTH_FILEOP_CLOSE:
            self.emit("KAUTH_FILEOP", MacOSEventType.EV_KAUTH_FILEOP, [self.cred.base, 0, action.value, self.vnode.base, path, params["flag"], 0])
        elif action == Kauth.KAUTH_FILEOP_WILL_RENAME:
            new_path = self.ql.os.heap.alloc(len(params["new_path"]) + 1)
            self.ql.mem.write(new_path, params["new_path"] + b"\x00")
            self.emit("KAUTH_FILEOP", MacOSEventType.EV_KAUTH_FILEOP, [self.cred.base, 0, action.value, self.vnode.base, path, new_path, 0])
        elif action == Kauth.KAUTH_FILEOP_RENAME:
            new_path = self.ql.os.heap.alloc(len(params["new_path"]) + 1)
            self.ql.mem.write(new_path, params["new_path"] + b"\x00")
            self.emit("KAUTH_FILEOP", MacOSEventType.EV_KAUTH_FILEOP, [self.cred.base, 0, action.value, path, new_path, 0, 0])
        elif action == Kauth.KAUTH_FILEOP_EXCHANGE:
            file1 = self.ql.os.heap.alloc(len(params["file_1"]) + 1)
            self.ql.mem.write(file1, params["file_1"] + b"\x00")
            file2 = self.ql.os.heap.alloc(len(params["file_2"]) + 1)
            self.ql.mem.write(file2, params["file_2"] + b"\x00")
            self.emit("KAUTH_FILEOP", MacOSEventType.EV_KAUTH_FILEOP, [self.cred.base, 0, action.value, file1, file2, 0, 0])
        elif action == Kauth.KAUTH_FILEOP_LINK:
            file1 = self.ql.os.heap.alloc(len(params["from"]) + 1)
            self.ql.mem.write(file1, params["from"] + b"\x00")
            file2 = self.ql.os.heap.alloc(len(params["to"]) + 1)
            self.ql.mem.write(file2, params["to"] + b"\x00")
            self.emit("KAUTH_FILEOP", MacOSEventType.EV_KAUTH_FILEOP, [self.cred.base, 0, action.value, file1, file2, 0, 0])
        elif action == Kauth.KAUTH_FILEOP_EXEC:
            self.emit("KAUTH_FILEOP", MacOSEventType.EV_KAUTH_FILEOP, [self.cred.base, 0, action.value, self.vnode.base, path, 0, 0])
        elif action == Kauth.KAUTH_FILEOP_DELETE:
            self.emit("KAUTH_FILEOP", MacOSEventType.EV_KAUTH_FILEOP, [self.cred.base, 0, action.value, self.vnode.base, path, 0, 0])

    # /bsd/kern/syscalls.master
    def syscall(self, sysnum, params):
        nsyscall, = struct.unpack("<Q", self.ql.mem.read(self.ql.loader.kernel_extrn_symbols_detail[b"_nsysent"]["n_value"], 8))
        sysent = self.ql.loader.kernel_local_symbols_detail[b"_sysent"]["n_value"]
        system_table = [sysent_t(self.ql, sysent + x * ctypes.sizeof(sysent_t)).loadFromMem() for x in range(nsyscall)]

        self.ql.uc.reg_write(UC_X86_REG_RAX, sysnum)
        reg_list = [UC_X86_REG_RDI, UC_X86_REG_RSI, UC_X86_REG_RDX, UC_X86_REG_R10, UC_X86_REG_R8, UC_X86_REG_R9]
        for idx, p in enumerate(params):
            self.ql.uc.reg_write(reg_list[idx], p)
        self.ql.os.savedrip=self.deadcode
        self.ql.run(begin=system_table[sysnum].sy_call.value)



#!/usr/bin/env python3
# 
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#

from abc import ABCMeta, abstractmethod
from gevent import Greenlet
from qiling.const import *
from .const import *

class QlThread(Greenlet):
    __metaclass__=ABCMeta

    def __init__(self, ql):
        super(QlThread, self).__init__()
        self.ql = ql
        self.log_file_fd = None

    def __str__(self):
        return f"QlThread {self.get_id()}"

# the common functions which are used in qiling core.
# these functions must be implemented in child class.

# like pthread_self(), return the id of currunt thread.
#    @abstractmethod
    def get_id(self):
        pass

# like pthread_exit(), terminate current thread.
#    @abstractmethod
    def exit(self):
        pass

    @abstractmethod
    def stop(self):
        pass




#!/usr/bin/env python3
#
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#

import gevent


from gevent.event import Event
from queue import Queue

class QlLinuxFutexManagement:
    
    FUTEX_BITSET_MATCH_ANY = 0xffffffff
    
    def __init__(self):
        self._wait_list = {}
    
    @property
    def wait_list(self):
        return self._wait_list
    
    def futex_wait(self, ql, uaddr, t, val, bitset=FUTEX_BITSET_MATCH_ANY):
        def _sched_wait_event(cur_thread):
            ql.log.debug(f"Wait for notifications.")
            event.wait()
        uaddr_value = ql.unpack32(ql.mem.read(uaddr, 4))
        if uaddr_value != val:
            ql.log.debug(f"uaddr: {hex(uaddr_value)} != {hex(val)}")
            return -1
        ql.emu_stop()
        if uaddr not in self.wait_list.keys():
            self.wait_list[uaddr] = Queue()
        event = Event()
        self.wait_list[uaddr].put((bitset, t, event))
        t.sched_cb = _sched_wait_event
        return 0
    
    def get_futex_wake_list(self, ql, addr, number, bitset=FUTEX_BITSET_MATCH_ANY):
        wakes = []
        if addr not in self.wait_list or number == 0:
            ql.log.debug(f"No thread at {hex(addr)}")
            return wakes
        thread_queue = self.wait_list[addr]
        if thread_queue.qsize() < number:
            number = thread_queue.qsize()
        for _ in range(number):
            orig_bitset, thread, event = thread_queue.get()
            if orig_bitset & bitset:
                wakes.append((thread, event))
        if self.wait_list[addr].qsize() == 0:
            del self.wait_list[addr]
        return wakes

    def futex_wake(self, ql, uaddr, t, number, bitset=FUTEX_BITSET_MATCH_ANY):
        def _sched_set_event(cur_thread):
            for t, e in wakes:
                ql.log.debug(f"Notify [Thread {t.get_id()}.")
                e.set()
            # Give up control.
            gevent.sleep(0)

        ql.emu_stop()
        wakes = self.get_futex_wake_list(ql, uaddr, number, bitset)
        t.sched_cb = _sched_set_event

        return len(wakes)



from .errhandlingapi import *
from .fibersapi import *
from .fileapi import *
from .handleapi import *
from .heapapi import *
from .ioapiset import *
from .libloaderapi import *
from .memoryapi import *
from .processenv import *
from .processthreadsapi import *
from .profileapi import *
from .stringapiset import *
from .synchapi import *
from .sysinfoapi import *
from .tlhelp32 import *
from .winbase import *
from .winnls import *
from .winnt import *
from .wow64apiset import *
from .interlockedapi import *
from .consoleapi2 import *
from .consoleapi3 import *
from .debugapi import *
from .psapi import *
